#!/usr/bin/env python3
# -*- coding: utf-8 -*-
#
# Copyright (C) 2017-2020  The Project X-Ray Authors.
#
# Use of this source code is governed by a ISC-style
# license that can be found in the LICENSE file or at
# https://opensource.org/licenses/ISC
#
# SPDX-License-Identifier: ISC

import sys, os, json
import pickle

from prjxray.util import OpenSafeFile

class MergeFind:
    def __init__(self):
        self.db = dict()

    def merge(self, a, b):
        a = self.find(a)
        b = self.find(b)
        if a != b:
            self.db[a] = b

    def find(self, a):
        if a in self.db:
            c = self.find(self.db[a])
            self.db[a] = c
            return c
        return a


def db_gen():
    print("Reading database..")

    with OpenSafeFile("%s/%s/tilegrid.json" % (os.getenv("XRAY_DATABASE_DIR"),
                                       os.getenv("XRAY_DATABASE")), "r") as f:
        tilegrid = json.load(f)

    with OpenSafeFile("%s/%s/tileconn.json" % (os.getenv("XRAY_DATABASE_DIR"),
                                       os.getenv("XRAY_DATABASE")), "r") as f:
        tileconn = json.load(f)

    type_to_tiles = dict()
    grid_to_tile = dict()
    nodes = MergeFind()

    for tile, tiledata in tilegrid.items():
        if tiledata["type"] not in type_to_tiles:
            type_to_tiles[tiledata["type"]] = list()
        type_to_tiles[tiledata["type"]].append(tile)
        grid_to_tile[(tiledata["grid_x"], tiledata["grid_y"])] = tile

    print("Processing tileconn..")

    for entry in tileconn:
        type_a, type_b = entry["tile_types"]
        for tile_a in type_to_tiles[type_a]:
            tiledata_a = tilegrid[tile_a]
            grid_a = (tiledata_a["grid_x"], tiledata_a["grid_y"])
            grid_b = (
                grid_a[0] + entry["grid_deltas"][0],
                grid_a[1] + entry["grid_deltas"][1])

            if grid_b not in grid_to_tile:
                continue

            tile_b = grid_to_tile[grid_b]
            tiledata_b = tilegrid[tile_b]

            if tiledata_b["type"] != type_b:
                continue

            for pair in entry["wire_pairs"]:
                nodes.merge((tile_a, pair[0]), (tile_b, pair[1]))

    print("Processing PIPs..")

    node_node_pip = dict()
    reverse_node_node = dict()

    for tile_type in ["int_l", "int_r"]:
        with OpenSafeFile("%s/%s/segbits_%s.db" %
                  (os.getenv("XRAY_DATABASE_DIR"), os.getenv("XRAY_DATABASE"),
                   tile_type), "r") as f:
            for line in f:
                _, dst, src = line.split()[0].split(".")
                for tile in type_to_tiles[tile_type.upper()]:
                    src_node = nodes.find((tile, src))
                    dst_node = nodes.find((tile, dst))
                    if src_node not in node_node_pip:
                        node_node_pip[src_node] = dict()
                    if dst_node not in reverse_node_node:
                        reverse_node_node[dst_node] = set()
                    node_node_pip[src_node][dst_node] = "%s.%s.%s" % (
                        tile, dst, src)
                    reverse_node_node[dst_node].add(src_node)

    return type_to_tiles, grid_to_tile, nodes, node_node_pip, reverse_node_node


def db_load():
    # Takes a while. Speed things up
    picklef = os.getenv('XRAY_DIR') + '/tools/simpleroute.p'
    if os.path.exists(picklef):
        #print('Pickle: load')
        db_res = pickle.load(open(picklef, 'rb'))
    else:
        #print('Pickle: rebuilding')
        db_res = db_gen()
        #print('Pickle: save')
        pickle.dump(db_res, open(picklef, 'wb'))
    return db_res


def route(args):
    type_to_tiles, grid_to_tile, nodes, node_node_pip, reverse_node_node = db_load(
    )

    active_pips = set()
    blocked_nodes = set()

    for argidx in range((len(args)) // 2):
        src_tile, src_wire = args[2 * argidx].split("/")
        dst_tile, dst_wire = args[2 * argidx + 1].split("/")

        src_node = nodes.find((src_tile, src_wire))
        dst_node = nodes.find((dst_tile, dst_wire))

        print("Routing %s -> %s:" % (src_node, dst_node))

        node_scores = dict()

        def write_scores(nodes, count):
            next_nodes = set()
            for n in nodes:
                if n in node_scores:
                    continue
                node_scores[n] = count
                if n == src_node:
                    return
                if n in reverse_node_node:
                    for nn in reverse_node_node[n]:
                        if nn not in node_scores and nn not in blocked_nodes:
                            next_nodes.add(nn)
            write_scores(next_nodes, count + 1)

        try:
            write_scores(set([dst_node]), 1)
        except RecursionError as e:
            raise Exception(
                "Could not find route for node %s" % (dst_node, )) from None
        print("  route length: %d" % node_scores[src_node])

        count = 0
        c = src_node
        blocked_nodes.add(c)
        print("  %4d: %s" % (count, c))

        score = node_scores[src_node]
        while c != dst_node:
            nn = None
            for n in node_node_pip[c].keys():
                if n in node_scores and node_scores[n] < score:
                    nn, score = n, node_scores[n]

            pip = node_node_pip[c][nn]
            active_pips.add(pip)
            print("        %s" % pip)

            count += 1
            c = nn
            blocked_nodes.add(c)
            print("  %4d: %s" % (count, c))

    print("====")
    pipnames = list()

    for pip in sorted(active_pips):
        tile, dst, src = pip.split(".")
        pipnames.append("%s/%s.%s->>%s" % (tile, tile[0:5], src, dst))

    print(
        "highlight_objects -color orange [get_nodes -of_objects [get_wires {%s}]]"
        % " ".join(["%s/%s" % n for n in sorted(blocked_nodes)]))
    print(
        "highlight_objects -color orange [get_pips {%s}]" % " ".join(pipnames))

    print("====")
    for pip in sorted(active_pips):
        print(pip)
    return active_pips


if __name__ == '__main__':
    if len(sys.argv) == 1 or (len(sys.argv) % 2) != 1:
        print()
        print("Usage: %s src1 dst1 [src2 dst2 [...]]" % sys.argv[0])
        print("Where entires as tile/wire")
        print()
        print(
            "Example: %s VBRK_X29Y140/VBRK_ER1BEG2 VFRAME_X47Y113/VFRAME_EL1BEG2"
            % sys.argv[0])
        print()
        sys.exit(1)
    route(sys.argv[1:])



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
#
# Copyright (C) 2017-2022  The Project X-Ray Authors.
#
# Use of this source code is governed by a ISC-style
# license that can be found in the LICENSE file or at
# https://opensource.org/licenses/ISC
#
# SPDX-License-Identifier: ISC

from prjxray.segmaker import Segmaker
import os
import os.path


def bitfilter(frame, word):
    if frame < 26:
        return False
    return True


def main():
    segmk = Segmaker("design.bits")

    tiledata = {}
    pipdata = {}
    ignpip = set()
    tile_ports = {}

    with open(os.path.join(os.getenv('FUZDIR'), '..', 'piplist', 'build',
                           'hclk_ioi', 'hclk_ioi.txt')) as f:
        for l in f:
            tile_type, dst, src = l.strip().split('.')
            if tile_type not in pipdata:
                pipdata[tile_type] = []
                tile_ports[tile_type] = set()

            pipdata[tile_type].append((src, dst))
            tile_ports[tile_type].add(src)
            tile_ports[tile_type].add(dst)

    print("Loading tags from design.txt.")
    with open("design.txt", "r") as f:
        for line in f:
            tile, pip, src, dst, pnum, pdir = line.split()

            if not tile.startswith('HCLK_IOI_'):
                continue

            pip_prefix, _ = pip.split(".")
            tile_from_pip, tile_type = pip_prefix.split('/')
            assert tile == tile_from_pip
            _, src = src.split("/")
            _, dst = dst.split("/")
            pnum = int(pnum)
            pdir = int(pdir)

            if tile not in tiledata:
                tiledata[tile] = {
                    "type": tile_type,
                    "pips": set(),
                    "srcs": set(),
                    "dsts": set()
                }

            tiledata[tile]["pips"].add((src, dst))
            tiledata[tile]["srcs"].add(src)
            tiledata[tile]["dsts"].add(dst)

            if pdir == 0:
                tiledata[tile]["srcs"].add(dst)
                tiledata[tile]["dsts"].add(src)

    for tile, pips_srcs_dsts in tiledata.items():
        tile_type = pips_srcs_dsts["type"]
        pips = pips_srcs_dsts["pips"]

        for src, dst in pipdata[tile_type]:
            if (src, dst) in ignpip:
                pass
            elif (src, dst) in pips:
                segmk.add_tile_tag(tile, "%s.%s" % (dst, src), 1)
            elif dst not in tiledata[tile]["dsts"]:
                segmk.add_tile_tag(tile, "%s.%s" % (dst, src), 0)

    segmk.compile(bitfilter=bitfilter)
    segmk.write()


if __name__ == "__main__":
    main()



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
#
# Copyright (C) 2017-2020  The Project X-Ray Authors.
#
# Use of this source code is governed by a ISC-style
# license that can be found in the LICENSE file or at
# https://opensource.org/licenses/ISC
#
# SPDX-License-Identifier: ISC
import os
import random
import itertools
from prjxray import util
from prjxray.db import Database
random.seed(int(os.getenv("SEED"), 16))


def gen_sites(lr):
    db = Database(util.get_db_root(), util.get_part())
    grid = db.grid()

    for tile_name in grid.tiles():
        loc = grid.loc_of_tilename(tile_name)
        gridinfo = grid.gridinfo_at_loc(loc)

        if gridinfo.tile_type[-1] != lr:
            continue

        sites = []
        for site_name, site_type in gridinfo.sites.items():
            if site_type in ['SLICEL', 'SLICEM']:
                sites.append(site_name)

        if not sites:
            continue

        print('// ', gridinfo.tile_type)

        yield sorted(sites)


def yield_bits(bits, nvals):
    for i in range(nvals):
        mask = (1 << i)
        yield int(bool(bits & mask))


NUM_IMUX_INPUTS = 2 * 6 * 4
NUM_TILES = 20


def emit_luts(choices, spec_idx, lr):
    for idx, sites in enumerate(itertools.islice(gen_sites(lr), 0, NUM_TILES)):

        cidx = idx + spec_idx * NUM_TILES
        if cidx < len(choices):
            bits = choices[cidx]
        else:
            bits = random.randint(0, 2**NUM_IMUX_INPUTS - 1)

        itr = yield_bits(bits, nvals=NUM_IMUX_INPUTS)

        for site in sites:
            for lut in range(4):
                print(
                    '''
    (* KEEP, DONT_TOUCH, LOC = "{site}" *)
    LUT6 {site}_lut{lut} (
        .I0({I0}),
        .I1({I1}),
        .I2({I2}),
        .I3({I3}),
        .I4({I4}),
        .I5({I5})
        );'''.format(
                        site=site,
                        lut=lut,
                        I0=next(itr),
                        I1=next(itr),
                        I2=next(itr),
                        I3=next(itr),
                        I4=next(itr),
                        I5=next(itr),
                    ))


def run():

    print('''
module top();
''')

    choices = util.gen_fuzz_choices(nvals=NUM_IMUX_INPUTS)
    spec_idx = util.specn() - 1

    emit_luts(choices, spec_idx, 'L')
    emit_luts(choices, spec_idx, 'R')

    print("endmodule")


if __name__ == '__main__':
    run()



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
#
# Copyright (C) 2017-2020  The Project X-Ray Authors.
#
# Use of this source code is governed by a ISC-style
# license that can be found in the LICENSE file or at
# https://opensource.org/licenses/ISC
#
# SPDX-License-Identifier: ISC

ports = {
    "PCIE_2_1": [
        ("CFGERRACSN", 1),
        ("CFGERRATOMICEGRESSBLOCKEDN", 1),
        ("CFGERRCORN", 1),
        ("CFGERRCPLABORTN", 1),
        ("CFGERRCPLTIMEOUTN", 1),
        ("CFGERRCPLUNEXPECTN", 1),
        ("CFGERRECRCN", 1),
        ("CFGERRINTERNALCORN", 1),
        ("CFGERRINTERNALUNCORN", 1),
        ("CFGERRLOCKEDN", 1),
        ("CFGERRMALFORMEDN", 1),
        ("CFGERRMCBLOCKEDN", 1),
        ("CFGERRNORECOVERYN", 1),
        ("CFGERRPOISONEDN", 1),
        ("CFGERRPOSTEDN", 1),
        ("CFGERRURN", 1),
        ("CFGFORCECOMMONCLOCKOFF", 1),
        ("CFGFORCEEXTENDEDSYNCON", 1),
        ("CFGINTERRUPTASSERTN", 1),
        ("CFGINTERRUPTN", 1),
        ("CFGINTERRUPTSTATN", 1),
        ("CFGMGMTRDENN", 1),
        ("CFGMGMTWRENN", 1),
        ("CFGMGMTWRREADONLYN", 1),
        ("CFGMGMTWRRW1CASRWN", 1),
        ("CFGPMFORCESTATEENN", 1),
        ("CFGPMHALTASPML0SN", 1),
        ("CFGPMHALTASPML1N", 1),
        ("CFGPMSENDPMETON", 1),
        ("CFGPMTURNOFFOKN", 1),
        ("CFGPMWAKEN", 1),
        ("CFGTRNPENDINGN", 1),
        ("CMRSTN", 1),
        ("CMSTICKYRSTN", 1),
        ("DBGSUBMODE", 1),
        ("DLRSTN", 1),
        ("DRPCLK", 1),
        ("DRPEN", 1),
        ("DRPWE", 1),
        ("FUNCLVLRSTN", 1),
        ("LL2SENDASREQL1", 1),
        ("LL2SENDENTERL1", 1),
        ("LL2SENDENTERL23", 1),
        ("LL2SENDPMACK", 1),
        ("LL2SUSPENDNOW", 1),
        ("LL2TLPRCV", 1),
        ("PIPECLK", 1),
        ("PIPERX0CHANISALIGNED", 1),
        ("PIPERX0ELECIDLE", 1),
        ("PIPERX0PHYSTATUS", 1),
        ("PIPERX0VALID", 1),
        ("PIPERX1CHANISALIGNED", 1),
        ("PIPERX1ELECIDLE", 1),
        ("PIPERX1PHYSTATUS", 1),
        ("PIPERX1VALID", 1),
        ("PIPERX2CHANISALIGNED", 1),
        ("PIPERX2ELECIDLE", 1),
        ("PIPERX2PHYSTATUS", 1),
        ("PIPERX2VALID", 1),
        ("PIPERX3CHANISALIGNED", 1),
        ("PIPERX3ELECIDLE", 1),
        ("PIPERX3PHYSTATUS", 1),
        ("PIPERX3VALID", 1),
        ("PIPERX4CHANISALIGNED", 1),
        ("PIPERX4ELECIDLE", 1),
        ("PIPERX4PHYSTATUS", 1),
        ("PIPERX4VALID", 1),
        ("PIPERX5CHANISALIGNED", 1),
        ("PIPERX5ELECIDLE", 1),
        ("PIPERX5PHYSTATUS", 1),
        ("PIPERX5VALID", 1),
        ("PIPERX6CHANISALIGNED", 1),
        ("PIPERX6ELECIDLE", 1),
        ("PIPERX6PHYSTATUS", 1),
        ("PIPERX6VALID", 1),
        ("PIPERX7CHANISALIGNED", 1),
        ("PIPERX7ELECIDLE", 1),
        ("PIPERX7PHYSTATUS", 1),
        ("PIPERX7VALID", 1),
        ("PLDIRECTEDLINKAUTON", 1),
        ("PLDIRECTEDLINKSPEED", 1),
        ("PLDIRECTEDLTSSMNEWVLD", 1),
        ("PLDIRECTEDLTSSMSTALL", 1),
        ("PLDOWNSTREAMDEEMPHSOURCE", 1),
        ("PLRSTN", 1),
        ("PLTRANSMITHOTRST", 1),
        ("PLUPSTREAMPREFERDEEMPH", 1),
        ("SYSRSTN", 1),
        ("TL2ASPMSUSPENDCREDITCHECK", 1),
        ("TL2PPMSUSPENDREQ", 1),
        ("TLRSTN", 1),
        ("TRNRDSTRDY", 1),
        ("TRNRFCPRET", 1),
        ("TRNRNPOK", 1),
        ("TRNRNPREQ", 1),
        ("TRNTCFGGNT", 1),
        ("TRNTDLLPSRCRDY", 1),
        ("TRNTECRCGEN", 1),
        ("TRNTEOF", 1),
        ("TRNTERRFWD", 1),
        ("TRNTSOF", 1),
        ("TRNTSRCDSC", 1),
        ("TRNTSRCRDY", 1),
        ("TRNTSTR", 1),
        ("USERCLK2", 1),
        ("USERCLK", 1),
        ("CFGERRAERHEADERLOG", 128),
        ("TRNTD", 128),
        ("CFGDEVID", 16),
        ("CFGSUBSYSID", 16),
        ("CFGSUBSYSVENDID", 16),
        ("CFGVENDID", 16),
        ("DRPDI", 16),
        ("PIPERX0DATA", 16),
        ("PIPERX1DATA", 16),
        ("PIPERX2DATA", 16),
        ("PIPERX3DATA", 16),
        ("PIPERX4DATA", 16),
        ("PIPERX5DATA", 16),
        ("PIPERX6DATA", 16),
        ("PIPERX7DATA", 16),
        ("CFGPMFORCESTATE", 2),
        ("DBGMODE", 2),
        ("PIPERX0CHARISK", 2),
        ("PIPERX1CHARISK", 2),
        ("PIPERX2CHARISK", 2),
        ("PIPERX3CHARISK", 2),
        ("PIPERX4CHARISK", 2),
        ("PIPERX5CHARISK", 2),
        ("PIPERX6CHARISK", 2),
        ("PIPERX7CHARISK", 2),
        ("PLDIRECTEDLINKCHANGE", 2),
        ("PLDIRECTEDLINKWIDTH", 2),
        ("TRNTREM", 2),
        ("CFGDSFUNCTIONNUMBER", 3),
        ("CFGFORCEMPS", 3),
        ("PIPERX0STATUS", 3),
        ("PIPERX1STATUS", 3),
        ("PIPERX2STATUS", 3),
        ("PIPERX3STATUS", 3),
        ("PIPERX4STATUS", 3),
        ("PIPERX5STATUS", 3),
        ("PIPERX6STATUS", 3),
        ("PIPERX7STATUS", 3),
        ("PLDBGMODE", 3),
        ("TRNFCSEL", 3),
        ("CFGMGMTDI", 32),
        ("TRNTDLLPDATA", 32),
        ("CFGMGMTBYTEENN", 4),
        ("CFGERRTLPCPLHEADER", 48),
        ("CFGAERINTERRUPTMSGNUM", 5),
        ("CFGDSDEVICENUMBER", 5),
        ("CFGPCIECAPINTERRUPTMSGNUM", 5),
        ("PL2DIRECTEDLSTATE", 5),
        ("PLDIRECTEDLTSSMNEW", 6),
        ("CFGDSN", 64),
        ("MIMRXRDATA", 68),
        ("MIMTXRDATA", 69),
        ("CFGDSBUSNUMBER", 8),
        ("CFGINTERRUPTDI", 8),
        ("CFGPORTNUMBER", 8),
        ("CFGREVID", 8),
        ("DRPADDR", 9),
        ("CFGMGMTDWADDR", 10),
    ]
}



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
#
# Copyright (C) 2017-2020  The Project X-Ray Authors.
#
# Use of this source code is governed by a ISC-style
# license that can be found in the LICENSE file or at
# https://opensource.org/licenses/ISC
#
# SPDX-License-Identifier: ISC
""" TileSegbitsAlias provides an alias from one tile type to another.
TileSegbitsAlias performs severals functions to achieve the alias:
 - Remaps tile type from the original tile type to the alias tile type
 - Offsets the bits from the original to the alias type
 - Renames sites from the original to the alias type
 - Filters bits outside of the alias.


"""

from prjxray import bitstream
from prjxray.grid_types import Bits
from prjxray.tile_segbits import read_ppips
from prjxray.util import OpenSafeFile


class TileSegbitsAlias(object):
    def __init__(self, db, tile_type, bits_map):
        # Name of tile_type that is using the alias
        self.tile_type = tile_type

        # Name of aliased tile_type
        self.alias_tile_type = None

        # BlockType -> BitAlias map
        self.alias = {}

        self.bits_map = bits_map

        # BlockType -> aliased Bits map
        self.alias_bits_map = {}

        # aliased site name to site name map
        self.sites_rev_map = {}

        for block_type in bits_map:
            self.alias[block_type] = bits_map[block_type].alias
            self.alias_bits_map[block_type] = Bits(
                base_address=bits_map[block_type].base_address,
                frames=bits_map[block_type].frames,
                offset=bits_map[block_type].offset -
                self.alias[block_type].start_offset,
                words=bits_map[block_type].words,
                alias=None,
            )

            if self.alias_tile_type is None:
                self.alias_tile_type = self.alias[block_type].tile_type
            else:
                assert self.alias_tile_type == self.alias[block_type].tile_type

            self.sites_rev_map[block_type] = {}
            for site, alias_site in self.alias[block_type].sites.items():
                assert alias_site not in self.sites_rev_map[block_type]
                self.sites_rev_map[block_type][alias_site] = site

        tile_db = db.tile_types[self.tile_type]
        self.ppips = {}

        if tile_db.ppips is not None:
            with OpenSafeFile(tile_db.ppips) as f:
                self.ppips = read_ppips(f)
        self.tile_segbits = db.get_tile_segbits(self.alias_tile_type)

    def map_feature_to_segbits(self, feature):
        """ Map from the output feature name to the aliased feature name. """
        parts = feature.split('.')

        assert parts[0] == self.tile_type
        parts[0] = self.alias_tile_type

        for block_type in self.alias:
            if len(parts) > 1 and parts[1] in self.alias[block_type].sites:
                parts[1] = self.alias[block_type].sites[parts[1]]

        return '.'.join(parts)

    def map_feature_from_segbits(self, feature):
        """ Map from the aliases feature name to the output feature name. """
        parts = feature.split('.')

        assert parts[0] == self.alias_tile_type
        parts[0] = self.tile_type

        for block_type in self.alias:
            if len(parts) > 1 and parts[1] in self.sites_rev_map[block_type]:
                parts[1] = self.sites_rev_map[block_type][parts[1]]

        return '.'.join(parts)

    def match_filter(self, block_type, query_bit):
        word = query_bit.word_bit // bitstream.WORD_SIZE_BITS
        real_word = word - self.alias[block_type].start_offset
        if real_word < 0 or real_word >= self.bits_map[block_type].words:
            return False

        return True

    def match_bitdata(self, block_type, bits, bitdata):
        alias_bits = self.alias_bits_map[block_type]

        for bits_found, alias_feature in self.tile_segbits.match_bitdata(
                block_type, alias_bits, bitdata,
                match_filter=self.match_filter):
            feature = self.map_feature_from_segbits(alias_feature)

            yield (bits_found, feature)

    def feature_to_bits(self, bits_map, feature, address=0):
        if feature in self.ppips:
            return

        alias_feature = self.map_feature_to_segbits(feature)
        for block_type, bit in self.tile_segbits.feature_to_bits(
                self.alias_bits_map, alias_feature, address):
            yield block_type, bit



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
#
# Copyright (C) 2017-2020  The Project X-Ray Authors.
#
# Use of this source code is governed by a ISC-style
# license that can be found in the LICENSE file or at
# https://opensource.org/licenses/ISC
#
# SPDX-License-Identifier: ISC

import os
import os.path
import re
from prjxray.segmaker import Segmaker


def src_has_active_bit(src):
    if re.match(r"^CLK_HROW_CK_INT_[01]_[01]", src) is not None:
        return False
    return True


def main():
    segmk = Segmaker("design.bits")

    tiledata = {}
    pipdata = {}
    clk_list = {}
    casco_list = {}
    ignpip = set()

    with open(os.path.join(os.getenv('FUZDIR'), '..', 'piplist', 'build',
                           'clk_hrow', 'clk_hrow_bot_r.txt')) as f:
        for l in f:
            tile_type, dst, src = l.strip().split('.')
            if tile_type not in pipdata:
                pipdata[tile_type] = []
                clk_list[tile_type] = set()
                casco_list[tile_type] = set()

            pipdata[tile_type].append((src, dst))

            if 'CASCO' in dst:
                casco_list[tile_type].add(dst)

            if dst.startswith('CLK_HROW_CK_MUX_OUT_'):
                clk_list[tile_type].add(src)

            if dst.startswith('CLK_HROW_BOT_R_CK_BUFG_'):
                if 'CASCIN' not in src:
                    clk_list[tile_type].add(src)

    with open(os.path.join(os.getenv('FUZDIR'), '..', 'piplist', 'build',
                           'clk_hrow', 'clk_hrow_top_r.txt')) as f:
        for l in f:
            tile_type, dst, src = l.strip().split('.')
            if tile_type not in pipdata:
                pipdata[tile_type] = []
                clk_list[tile_type] = set()
                casco_list[tile_type] = set()

            pipdata[tile_type].append((src, dst))

            if 'CASCO' in dst:
                casco_list[tile_type].add(dst)

            if dst.startswith('CLK_HROW_CK_MUX_OUT_'):
                clk_list[tile_type].add(src)

            if dst.startswith('CLK_HROW_TOP_R_CK_BUFG_'):
                if 'CASCIN' not in src:
                    clk_list[tile_type].add(src)

    print("Loading tags from design.txt.")
    with open("design.txt", "r") as f:
        for line in f:
            tile, pip, src, dst, pnum, pdir = line.split()

            if not tile.startswith('CLK_HROW'):
                continue

            pip_prefix, _ = pip.split(".")
            tile_from_pip, tile_type = pip_prefix.split('/')
            assert tile == tile_from_pip
            _, src = src.split("/")
            _, dst = dst.split("/")
            pnum = int(pnum)
            pdir = int(pdir)

            if tile not in tiledata:
                tiledata[tile] = {
                    "type": tile_type,
                    "pips": set(),
                    "srcs": set(),
                    "dsts": set()
                }

            tiledata[tile]["pips"].add((src, dst))
            tiledata[tile]["srcs"].add(src)
            tiledata[tile]["dsts"].add(dst)

            if pdir == 0:
                tiledata[tile]["srcs"].add(dst)
                tiledata[tile]["dsts"].add(src)

            if pnum == 1 or pdir == 0:
                ignpip.add((src, dst))

    active_gclks = {}
    active_clks = {}

    for tile, pips_srcs_dsts in tiledata.items():
        tile_type = pips_srcs_dsts["type"]
        pips = pips_srcs_dsts["pips"]

        if tile not in active_clks:
            active_clks[tile] = set()

        for src, dst in pips_srcs_dsts["pips"]:
            active_clks[tile].add(src)

            if 'GCLK' in src:
                if src not in active_gclks:
                    active_gclks[src] = set()

                active_gclks[src].add(tile)

        for src, dst in pipdata[tile_type]:
            if (src, dst) in ignpip:
                pass
            elif (src, dst) in pips:
                segmk.add_tile_tag(tile, "%s.%s" % (dst, src), 1)
            elif dst not in tiledata[tile]["dsts"]:
                segmk.add_tile_tag(tile, "%s.%s" % (dst, src), 0)

    for tile_type, srcs in clk_list.items():
        for tile, pips_srcs_dsts in tiledata.items():
            for src in srcs:
                #Don't solve fake features
                if not src_has_active_bit(src):
                    continue
                if 'GCLK' not in src:
                    active = src in active_clks[tile]
                    segmk.add_tile_tag(tile, '{}_ACTIVE'.format(src), active)
                else:
                    if src not in active_gclks:
                        segmk.add_tile_tag(tile, '{}_ACTIVE'.format(src), 0)
                    elif tile in active_gclks[src]:
                        segmk.add_tile_tag(tile, '{}_ACTIVE'.format(src), 1)

    segmk.compile()
    segmk.write()


if __name__ == '__main__':
    main()



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
#
# Copyright (C) 2017-2020  The Project X-Ray Authors.
#
# Use of this source code is governed by a ISC-style
# license that can be found in the LICENSE file or at
# https://opensource.org/licenses/ISC
#
# SPDX-License-Identifier: ISC

from prjxray import xjson
import json
import util as localutil
import os.path


def check_frames(tagstr, addrlist):
    frames = set()
    for addrstr in addrlist:
        frame = parse_addr(addrstr, get_base_frame=True)
        frames.add(frame)
    assert len(frames) == 1, (
        "{}: More than one base address".format(tagstr), map(hex, frames))


def parse_addr(line, only_frame=False, get_base_frame=False):
    # 00020027_003_03
    line = line.split("_")
    frame = int(line[0], 16)
    wordidx = int(line[1], 10)
    bitidx = int(line[2], 10)

    if get_base_frame:
        delta = frame % 128
        frame -= delta
        return frame

    return frame, wordidx, bitidx


def load_db(fn):
    for l in open(fn, "r"):
        l = l.strip()
        # FIXME: add offset to name
        # IOB_X0Y101.DFRAME:27.DWORD:3.DBIT:3 00020027_003_03
        parts = l.split(' ')
        tagstr = parts[0]
        addrlist = parts[1:]
        assert not any(s == '<const0>' for s in addrlist), (fn, l)
        check_frames(tagstr, addrlist)
        # Take the first address in the list
        frame, wordidx, bitidx = parse_addr(addrlist[0])

        bitidx_up = False

        tparts = tagstr.split('.')
        tile = tparts[0]

        for part in tparts[1:]:
            # Auto align the frame address to the next lowest multiple of 0x80.
            if part == 'AUTO_FRAME':
                frame -= (frame % 0x80)
                continue

            k, v = part.split(':')
            if k == "DFRAME":
                frame -= int(v, 16)
            elif k == "DWORD":
                wordidx -= int(v, 10)
            elif k == "DBIT":
                bitidx -= int(v, 10)
                bitidx_up = True
            else:
                assert 0, (l, part)

        # XXX: maybe just ignore bitidx and always set to 0 instead of allowing explicit
        # or detect the first delta auto and assert they are all the same
        if not bitidx_up:
            bitidx = 0
        assert bitidx == 0, l
        assert frame % 0x80 == 0, "Unaligned frame at 0x%08X" % frame
        yield (tile, frame, wordidx)


def run(fn_in, fn_out, verbose=False):
    database = json.load(open(fn_in, "r"))

    # Load a map of sites to base addresses
    # Need to figure out the
    # FIXME: generate frames from part file (or equivilent)
    # See https://github.com/SymbiFlow/prjxray/issues/327
    # FIXME: generate words from pitch
    int_frames, int_words = localutil.get_int_params()
    tdb_fns = [
        ("iob", 42, 4),
        ("iob18", 42, 4),
        ("ioi", 42, 4),
        ("ioi18", 42, 4),
        ("mmcm", 30, 49),
        ("pll", 30, 27),
        ("monitor", 30, 101),
        ("bram", 28, 10),
        ("bram_block", 128, 10),
        ("clb", 36, 2),
        ("cfg", 30, 101),
        ("dsp", 28, 10),
        ("clk_hrow", 30, 18),
        ("clk_bufg", 30, 8),
        ("hclk_cmt", 30, 10),
        ("hclk_ioi", 42, 1),
        ("pcie", 36, 101),
        ("gtp_common", 32, 101),
        ("gtp_channel", 32, 22),
        ("gtx_common", 32, 101),
        ("gtx_channel", 32, 22),
        ("clb_int", int_frames, int_words),
        ("iob_int", int_frames, int_words),
        ("iob18_int", int_frames, int_words),
        ("bram_int", int_frames, int_words),
        ("dsp_int", int_frames, int_words),
        ("fifo_int", int_frames, int_words),
        ("ps7_int", int_frames, int_words),
        ("cfg_int", int_frames, int_words),
        ("monitor_int", int_frames, int_words),
        ("orphan_int_column", int_frames, int_words),
        ("gtp_int_interface", int_frames, int_words),
        ("gtx_int_interface", int_frames, int_words),
        ("pcie_int_interface", int_frames, int_words),
    ]

    tile_frames_map = localutil.TileFrames()
    for (subdir, frames, words) in tdb_fns:
        tdb_fn = os.path.join(
            subdir, 'build_{}'.format(os.environ['XRAY_PART']),
            'segbits_tilegrid.tdb')
        if not os.path.exists(tdb_fn):
            verbose and print('Skipping {}, file not found!'.format(tdb_fn))
            continue

        for (tile, frame, wordidx) in load_db(tdb_fn):
            tilej = database[tile]
            verbose and print("Add %s %08X_%03u" % (tile, frame, wordidx))
            localutil.add_tile_bits(
                tile, tilej, frame, wordidx, frames, words, tile_frames_map)

    # Save
    xjson.pprint(open(fn_out, "w"), database)


def main():
    import argparse

    parser = argparse.ArgumentParser(
        description="Annotate tilegrid addresses using solved base addresses")
    parser.add_argument("--verbose", action="store_true", help="")
    parser.add_argument("--fn-in", required=True, help="")
    parser.add_argument("--fn-out", required=True, help="")
    args = parser.parse_args()

    run(args.fn_in, args.fn_out, verbose=args.verbose)


if __name__ == "__main__":
    main()



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
#
# Copyright (C) 2017-2020  The Project X-Ray Authors.
#
# Use of this source code is governed by a ISC-style
# license that can be found in the LICENSE file or at
# https://opensource.org/licenses/ISC
#
# SPDX-License-Identifier: ISC
import sys
import random
import re


def top_harness(DIN_N, DOUT_N, f=sys.stdout):
    f.write(
        '''
module top(input clk, stb, di, output do);
    localparam integer DIN_N = %d;
    localparam integer DOUT_N = %d;

    reg [DIN_N-1:0] din;
    wire [DOUT_N-1:0] dout;

    reg [DIN_N-1:0] din_shr;
    reg [DOUT_N-1:0] dout_shr;

    always @(posedge clk) begin
        din_shr <= {din_shr, di};
        dout_shr <= {dout_shr, din_shr[DIN_N-1]};
        if (stb) begin
            din <= din_shr;
            dout_shr <= dout;
        end
    end

    assign do = dout_shr[DOUT_N-1];

    roi roi (
        .clk(clk),
        .din(din),
        .dout(dout)
    );
endmodule
''' % (DIN_N, DOUT_N))


def instance(mod, name, ports, params={}, sort=True, string_buffer=sys.stdout):
    # TODO: make this print nicer
    tosort = sorted if sort else lambda x: x
    print('    %s' % mod, file=string_buffer)
    if len(params):
        print('        #(', file=string_buffer)
        for i, (paramk, paramv) in enumerate(tosort(params.items())):
            comma = '' if i == len(params) - 1 else ','
            print(
                '            .%s(%s)%s' % (paramk, paramv, comma),
                file=string_buffer)
        print('        )', file=string_buffer)
    print('        %s (' % name, file=string_buffer)
    for i, (portk, portv) in enumerate(tosort(ports.items())):
        comma = '' if i == len(ports) - 1 else ','
        print(
            '            .%s(%s)%s' % (portk, portv, comma),
            file=string_buffer)
    print('        );', file=string_buffer)


def quote(s):
    return '"' + s + '"'


def unquote(s):
    assert s[0] == '"' and s[-1] == '"'
    return s[1:-1]


def to_int(s):
    value = 0

    match = re.search(r'^(\d+)\'([sS]*)([bBoOdDhH])(.*)', str(s))

    if match:
        width = int(match.group(1))
        signed = match.group(2)
        radix = match.group(3)
        value = match.group(4)

        # Convert to int type
        if re.match(r'[bB]', radix):
            value = int(value, 2)
        elif re.match(r'[oO]', radix):
            value = int(value, 8)
        elif re.match(r'[dD]', radix):
            value = int(value, 10)
        elif re.match(r'[hH]', radix):
            value = int(value, 16)
        else:
            raise ValueError('Don\'t know how to interpret input %s' % (s))

        # Truncate to `width` bits
        value &= 2**width - 1
    else:
        value = int(s)

    return value


def parsei(s):
    if s == "1'b0":
        return 0
    elif s == "1'b1":
        return 1
    else:
        assert 0, 'FIXME'


def parse_bitstr(s):
    n, postfix = s.split("'")
    n = int(n)
    assert postfix[0] == 'b'
    bitstr = postfix[1:]
    assert len(bitstr) == n
    return [int(x) for x in bitstr]


def vrandbit():
    if random.randint(0, 1):
        return "1'b1"
    else:
        return "1'b0"


def vrandbits(n):
    ret = "%u'b" % n
    for _i in range(n):
        ret += str(random.randint(0, 1))
    return ret



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
#
# Copyright (C) 2017-2020  The Project X-Ray Authors.
#
# Use of this source code is governed by a ISC-style
# license that can be found in the LICENSE file or at
# https://opensource.org/licenses/ISC
#
# SPDX-License-Identifier: ISC
from __future__ import print_function
import prjxray.db
import prjxray.util
import argparse


def quick_test(db_root, part):
    db = prjxray.db.Database(db_root, part)
    g = db.grid()

    # Verify that we have some tile information for every tile in grid.
    tile_types_in_grid = set(
        g.gridinfo_at_loc(loc).tile_type for loc in g.tile_locations())
    tile_types_in_db = set(db.get_tile_types())
    site_types = set(db.get_site_types())
    assert len(tile_types_in_grid - tile_types_in_db) == 0

    # Verify that all tile types can be loaded.
    for tile_type in db.get_tile_types():
        tile = db.get_tile_type(tile_type)
        wires = tile.get_wires()
        for site in tile.get_sites():
            assert site.type in site_types
            site_type = db.get_site_type(site.type)
            site_pins = site_type.get_site_pins()
            for site_pin in site.site_pins:
                if site_pin.wire is not None:
                    assert site_pin.wire in wires, (site_pin.wire, )

                assert site_pin.name in site_pins

        for pip in tile.get_pips():
            assert pip.net_to in wires
            assert pip.net_from in wires

    for loc in g.tile_locations():
        gridinfo = g.gridinfo_at_loc(loc)
        assert gridinfo.tile_type in db.get_tile_types()
        for site_name, site_type in gridinfo.sites.items():
            assert site_type in site_types

        tile = db.get_tile_type(gridinfo.tile_type)

        instance_sites = list(tile.get_instance_sites(gridinfo))
        assert len(instance_sites) == len(tile.get_sites())


def main():
    parser = argparse.ArgumentParser(
        description="Runs a sanity check on a prjxray database.")

    util.db_root_arg(parser)
    util.part_arg(parser)

    args = parser.parse_args()

    quick_test(args.db_root, args.part)


if __name__ == '__main__':
    main()



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
#
# Copyright (C) 2017-2020  The Project X-Ray Authors.
#
# Use of this source code is governed by a ISC-style
# license that can be found in the LICENSE file or at
# https://opensource.org/licenses/ISC
#
# SPDX-License-Identifier: ISC

import argparse
import json

from sdf_timing import sdfparse
from prjxray.util import OpenSafeFile


def merge(timings_list, site):

    merged_timings = dict()

    for timings in timings_list:
        divider = '/'
        if 'divider' in timings['header']:
            divider = timings['header']['divider']

        for cell in timings['cells']:
            for cell_instance in timings['cells'][cell]:
                if site in cell_instance.split(divider):
                    if 'cells' not in merged_timings:
                        merged_timings['cells'] = dict()
                    if cell not in merged_timings['cells']:
                        merged_timings['cells'][cell] = dict()
                    if cell_instance not in merged_timings['cells'][cell]:
                        merged_timings['cells'][cell][cell_instance] = dict()

                    if cell_instance in merged_timings['cells'][cell][
                            cell_instance]:
                        assert merged_timings['cells'][cell][cell_instance] == \
                               timings['cells'][cell][cell_instance],          \
                               "Attempting to merge differing cells"

                    merged_timings['cells'][cell][cell_instance] = timings[
                        'cells'][cell][cell_instance]

    return merged_timings


def main():

    parser = argparse.ArgumentParser()
    parser.add_argument(
        '--sdfs', nargs='+', type=str, help="List of sdf files to merge")
    parser.add_argument('--site', type=str, help="Site we want to merge")
    parser.add_argument('--json', type=str, help="Debug JSON")
    parser.add_argument('--out', type=str, help="Merged sdf name")

    args = parser.parse_args()

    timings_list = list()

    for sdf in args.sdfs:
        with OpenSafeFile(sdf, 'r') as fp:
            timing = sdfparse.parse(fp.read())
            timings_list.append(timing)

    merged_sdf = merge(timings_list, args.site)
    with OpenSafeFile(args.out, 'w') as fp:
        fp.write(sdfparse.emit(merged_sdf, timescale='1ns'))

    if args.json is not None:
        with OpenSafeFile(args.json, 'w') as fp:
            json.dump(merged_sdf, fp, indent=4, sort_keys=True)


if __name__ == '__main__':
    main()



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
#
# Copyright (C) 2017-2020  The Project X-Ray Authors.
#
# Use of this source code is governed by a ISC-style
# license that can be found in the LICENSE file or at
# https://opensource.org/licenses/ISC
#
# SPDX-License-Identifier: ISC

import sys

from prjxray.segmaker import Segmaker

segmk = Segmaker("design_%s.bits" % sys.argv[1])

print("Loading tags from design_%s.txt." % sys.argv[1])
with open("design_%s.txt" % sys.argv[1], "r") as f:
    for line in f:
        line = line.split()
        site = line[0]
        bel = line[1]
        init = int(line[2][4:], 16)

        for i in range(64):
            bitname = "%s.INIT[%02d]" % (bel, i)
            bitname = bitname.replace("6LUT", "LUT")
            segmk.add_site_tag(site, bitname, ((init >> i) & 1) != 0)

segmk.compile()
segmk.write(sys.argv[1])



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
#
# Copyright (C) 2017-2020  The Project X-Ray Authors.
#
# Use of this source code is governed by a ISC-style
# license that can be found in the LICENSE file or at
# https://opensource.org/licenses/ISC
#
# SPDX-License-Identifier: ISC
'''
Script for adding the IO Banks information to the Part's generated JSON.
'''
import argparse
import json


def main(argv):
    with open(args.part_json) as json_file, open(
            args.iobanks_info) as iobanks_info:
        part_data = json.load(json_file)
        json_file.close()
        iobank_data = dict()
        for iobank in iobanks_info:
            iobank = iobank.strip()
            bank, coordinates = iobank.split(",")
            iobank_data[bank] = coordinates
        iobanks_info.close()
        if len(iobank_data) > 0:
            part_data["iobanks"] = iobank_data
        print(json.dumps(part_data, indent=4))


if __name__ == "__main__":
    parser = argparse.ArgumentParser()
    parser.add_argument('--part_json', help='Input json')
    parser.add_argument('--iobanks_info', help='Input IO Banks info file')
    args = parser.parse_args()
    main(args)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
#
# Copyright (C) 2017-2020  The Project X-Ray Authors.
#
# Use of this source code is governed by a ISC-style
# license that can be found in the LICENSE file or at
# https://opensource.org/licenses/ISC
#
# SPDX-License-Identifier: ISC

import json

from prjxray.segmaker import Segmaker


def write_ram_ext_tags(segmk, tile_param):
    for param in ["RAM_EXTENSION_A", "RAM_EXTENSION_B"]:
        set_val = tile_param[param]
        for opt in ["LOWER"]:
            segmk.add_site_tag(
                tile_param['site'], "{}_{}".format(param, opt), set_val == opt)
        segmk.add_site_tag(
            tile_param['site'], "{}_NONE_OR_UPPER".format(param),
            set_val != "LOWER")


def main():
    segmk = Segmaker("design.bits")

    print("Loading tags")
    with open('params.json') as f:
        params = json.load(f)

    for tile_param in params:
        if tile_param['BRAM36_IN_USE']:
            write_ram_ext_tags(segmk, tile_param)

            segmk.add_site_tag(
                tile_param['site'], 'EN_ECC_READ', tile_param['EN_ECC_READ'])
            segmk.add_site_tag(
                tile_param['site'], 'EN_ECC_WRITE', tile_param['EN_ECC_WRITE'])

    for ab in 'ab':
        for rw in 'rw':
            if rw == 'r':
                dir = 'READ'
            else:
                dir = 'WRITE'

            width = tile_param['bram36_{}{}_width'.format(rw, ab)]
            tag = 'BRAM36_{}_WIDTH_{}_1'.format(dir, ab.upper())
            if width == 1:
                segmk.add_site_tag(
                    tile_param['site'], tag, tile_param['BRAM36_IN_USE'])

    segmk.compile()
    segmk.write()


if __name__ == '__main__':
    main()



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
#
# Copyright (C) 2017-2020  The Project X-Ray Authors.
#
# Use of this source code is governed by a ISC-style
# license that can be found in the LICENSE file or at
# https://opensource.org/licenses/ISC
#
# SPDX-License-Identifier: ISC

import argparse
import hashlib
import os
import parse as format_parser
import subprocess
import sys

"""Module for generating the Info.md file found in the database directory."""

info_md_header = """
# Details

Last updated on {human_date} ({iso8601_date}).

Created using [Project X-Ray](https://github.com/SymbiFlow/prjxray) version [{commit_hash_short}](https://github.com/SymbiFlow/prjxray/commit/{commit_hash_long}).

Latest commit was;
```
{commit_latest}
```

"""

info_md_section = """

## Database for [{part_line}]({part_line}/)

### Settings

Created using following [settings/{part_line}.sh (sha256: {settings_sha256})](https://github.com/SymbiFlow/prjxray/blob/{commit_hash_long}/settings/{part_line}.sh)
```shell
{settings_contents}
```

### [Results]({part_line}/)

Results have checksums;

"""

info_md_file = " * [`{file_sha256}  ./{file_short_path}`](./{file_short_path})\n"


def sha256(s):
    m = hashlib.sha256()
    m.update(s)
    return m.hexdigest()


def sha256_file(p):
    return sha256(open(p, 'rb').read())


def run(c):
    o = subprocess.check_output(c, shell=True)
    return o.decode('utf-8').strip()


def main(argv):

    parser = argparse.ArgumentParser()
    parser.add_argument(
        '--keep',
        default=False,
        action="store_true",
        help="""\
Keep the existing commit information.
""")
    args = parser.parse_args()

    info_md_filename = os.path.join('database', 'Info.md')
    assert os.path.exists(info_md_filename), info_md_filename

    info_md = []

    info_md.append(open('database/README.md').read())

    v = {}
    v['human_date'] = run('TZ=UTC date')
    v['iso8601_date'] = run('TZ=UTC date --iso-8601=seconds')
    if not args.keep:
        v['commit_latest'] = run('git log -1')
        v['commit_hash_short'] = run('git log -1 --pretty=%h')
        v['commit_hash_long'] = run('git log -1 --pretty=%H')
    else:
        with open(info_md_filename) as f:
            result = format_parser.parse(
                '{before}' + info_md_header + '{after}', f.read())
        assert result
        assert result['human_date']
        assert result['iso8601_date']
        v['commit_latest'] = result['commit_latest']
        v['commit_hash_short'] = result['commit_hash_short']
        v['commit_hash_long'] = result['commit_hash_long']

    info_md.append(info_md_header.format(**v))

    for part_line in sorted(os.listdir('database')):
        if part_line.startswith('.'):
            continue
        part_path = os.path.join('database', part_line)

        if not os.path.isdir(part_path):
            continue

        files = list(os.listdir(part_path))
        files.sort()

        settings_path = os.path.join('settings', part_line + '.sh')
        settings_raw = open(settings_path, 'rb').read()

        w = {}
        w['commit_hash_long'] = v['commit_hash_long']
        w['part_line'] = part_line
        w['settings_contents'] = settings_raw.decode('utf-8')
        w['settings_sha256'] = sha256(settings_raw)

        info_md.append(info_md_section.format(**w))

        files = []
        for dirpath, dirnames, filenames in os.walk(part_path):
            for f in filenames:
                files.append(os.path.join(dirpath, f))

        files.sort()
        for p in files:
            x = {}
            x['file_real_path'] = './' + p
            x['file_short_path'] = os.path.join(
                part_line, os.path.relpath(p, part_path))
            x['file_sha256'] = sha256_file(p)
            info_md.append(info_md_file.format(**x))

    with open(info_md_filename, 'w') as f:
        f.write("".join(info_md))

    return 0


if __name__ == "__main__":
    sys.exit(main(sys.argv))



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
#
# Copyright (C) 2017-2020  The Project X-Ray Authors.
#
# Use of this source code is governed by a ISC-style
# license that can be found in the LICENSE file or at
# https://opensource.org/licenses/ISC
#
# SPDX-License-Identifier: ISC
import os
import random
import itertools
random.seed(int(os.getenv("SEED"), 16))
from prjxray import util
from prjxray.db import Database


def gen_dsps():
    db = Database(util.get_db_root(), util.get_part())
    grid = db.grid()
    for tile_name in sorted(grid.tiles()):
        loc = grid.loc_of_tilename(tile_name)
        gridinfo = grid.gridinfo_at_loc(loc)

        sites = []
        for site_name, site_type in gridinfo.sites.items():
            if site_type in ['DSP48E1']:
                sites.append(site_name)

        sites.sort()

        if len(sites) == 0:
            continue

        if gridinfo.tile_type[-1] == 'L':
            int_grid_x = loc.grid_x + 2
            int_tile_type = 'INT_L'
        else:
            int_grid_x = loc.grid_x - 2
            int_tile_type = 'INT_R'

        int_tile_locs = [
            (int_grid_x, loc.grid_y),
            (int_grid_x, loc.grid_y - 1),
            (int_grid_x, loc.grid_y - 2),
            (int_grid_x, loc.grid_y - 3),
            (int_grid_x, loc.grid_y - 4),
        ]

        int_tiles = []
        for int_tile_loc in int_tile_locs:
            int_gridinfo = grid.gridinfo_at_loc(int_tile_loc)
            assert int_gridinfo.tile_type == int_tile_type, (
                int_gridinfo.tile_type, int_tile_type)

            int_tiles.append(grid.tilename_at_loc(int_tile_loc))

        yield tile_name, sites, int_tiles


def write_params(params):
    pinstr = 'tile,val\n'
    for tile, (val, ) in sorted(params.items()):
        pinstr += '%s,%s\n' % (tile, val)
    open('params.csv', 'w').write(pinstr)


def run():
    print('''
module top();
    ''')

    params = {}

    sites = list(gen_dsps())
    fuzz_iter = iter(util.gen_fuzz_states(len(sites) * 5))
    for tile_name, dsp_sites, int_tiles in sites:
        int_tiles.reverse()

        # Each DSP tile has 5 INT tiles.
        # The following feature is used to toggle a one PIP in each INT tile
        #
        # For a DSP, there are the following INT tiles and the feature
        # to toggle
        #
        # INT_L_X34Y89/INT_L.GFAN1->>IMUX_L30 :: D1.C36 = 0
        # INT_L_X34Y89/INT_L.GFAN1->>IMUX_L28 :: D1.CARRYINSEL1 = toggle
        #
        # INT_L_X34Y88/INT_L.GFAN1->>IMUX_L30 :: D0.CARRYINSEL0 = toggle
        #
        # INT_L_X34Y87/INT_L.GFAN1->>IMUX_L5  :: D1.A29 = 0
        # INT_L_X34Y87/INT_L.GFAN1->>IMUX_L14 :: D1.B8 = toggle
        #
        # INT_L_X34Y86/INT_L.GFAN1->>IMUX_L30 :: D1.B4 = 0
        # INT_L_X34Y86/INT_L.GFAN1->>IMUX_L28 :: D1.B6 = toggle
        #
        # INT_L_X34Y85/INT_L.GFAN1->>IMUX_L30 :: Dark Green :: D1.C20 = 0
        # INT_L_X34Y85/INT_L.GFAN1->>IMUX_L28 :: Color 10 :: D1.B2 = toggle
        (d1_carryinsel1, d0_carryinsel0, d1_b8, d1_b6,
         d1_b2) = itertools.islice(fuzz_iter, 5)
        params[int_tiles[0]] = (d1_carryinsel1, )
        params[int_tiles[1]] = (d0_carryinsel0, )
        params[int_tiles[2]] = (d1_b8, )
        params[int_tiles[3]] = (d1_b6, )
        params[int_tiles[4]] = (d1_b2, )

        print(
            '''
            wire [6:0] {dsp_site0}_opmode;
            wire [2:0] {dsp_site0}_carryinsel;

            wire [29:0] {dsp_site1}_a;
            wire [17:0] {dsp_site1}_b;
            wire [47:0] {dsp_site1}_c;
            wire [6:0] {dsp_site1}_opmode;
            wire [2:0] {dsp_site1}_carryinsel;

            // INT 0, {int0}
            assign {dsp_site1}_c[36] = 0;
            assign {dsp_site1}_carryinsel[1] = {d1_carryinsel1};

            // INT 1, {int1}
            assign {dsp_site0}_carryinsel[0] = {d0_carryinsel0};

            // INT 2, {int2}
            assign {dsp_site1}_a[29] = 0;
            assign {dsp_site1}_b[8] = {d1_b8};

            // INT 3, {int3}
            assign {dsp_site1}_b[4] = 0;
            assign {dsp_site1}_b[6] = {d1_b6};

            // INT 4, {int4}
            assign {dsp_site1}_c[20] = 0;
            assign {dsp_site1}_b[2] = {d1_b2};

            (* KEEP, DONT_TOUCH, LOC = "{dsp_site0}" *)
            DSP48E1 #(
                    .OPMODEREG(0),
                    .CREG(0)
                ) dsp_{dsp_site0} (
                    .OPMODE({dsp_site0}_opmode),
                    .CARRYINSEL({dsp_site0}_carryinsel)
                    );

            (* KEEP, DONT_TOUCH, LOC = "{dsp_site1}" *)
            DSP48E1 #(
                    .OPMODEREG(0),
                    .CREG(0)
                ) dsp_{dsp_site1} (
                    .OPMODE({dsp_site1}_opmode),
                    .CARRYINSEL({dsp_site1}_carryinsel),
                    .A({dsp_site1}_a),
                    .B({dsp_site1}_b),
                    .C({dsp_site1}_c)
                    );
'''.format(
                dsp_site0=dsp_sites[0],
                dsp_site1=dsp_sites[1],
                d1_carryinsel1=d1_carryinsel1,
                d0_carryinsel0=d0_carryinsel0,
                d1_b8=d1_b8,
                d1_b6=d1_b6,
                d1_b2=d1_b2,
                int0=int_tiles[0],
                int1=int_tiles[1],
                int2=int_tiles[2],
                int3=int_tiles[3],
                int4=int_tiles[4],
            ))

    print("endmodule")
    write_params(params)


if __name__ == '__main__':
    run()



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
#
# Copyright (C) 2017-2020  The Project X-Ray Authors.
#
# Use of this source code is governed by a ISC-style
# license that can be found in the LICENSE file or at
# https://opensource.org/licenses/ISC
#
# SPDX-License-Identifier: ISC
""" Tool for generating Vivavo commands to highlight objects from a FASM file.

Currently this tool only highlights pips directly referenced in the FASM file.
"""

from __future__ import print_function
import os.path
import fasm
from prjxray import db
from prjxray import util


def main():
    import argparse

    parser = argparse.ArgumentParser(
        description=
        'Outputs a Vivavo highlight_objects command from a FASM file.')

    util.db_root_arg(parser)
    util.part_arg(parser)
    parser.add_argument('fn_in', help='Input FPGA assembly (.fasm) file')

    args = parser.parse_args()
    database = db.Database(args.db_root, args.part)
    grid = database.grid()

    def inner():
        for line in fasm.parse_fasm_filename(args.fn_in):
            if not line.set_feature:
                continue

            parts = line.set_feature.feature.split('.')
            tile = parts[0]
            gridinfo = grid.gridinfo_at_tilename(tile)

            tile_type = database.get_tile_type(gridinfo.tile_type)

            for pip in tile_type.pips:
                if pip.net_from == parts[2] and pip.net_to == parts[1]:
                    yield '{}/{}'.format(tile, pip.name)

    print(
        'highlight_objects [concat {}]'.format(
            ' '.join('[get_pips {}]'.format(pip) for pip in inner())))


if __name__ == '__main__':
    main()



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
#
# Copyright (C) 2017-2020  The Project X-Ray Authors.
#
# Use of this source code is governed by a ISC-style
# license that can be found in the LICENSE file or at
# https://opensource.org/licenses/ISC
#
# SPDX-License-Identifier: ISC
'''
Extract the frames from the output of the prjxray bitread tool.
'''

import sys
import re


def extract_frames(args):
    """ Extract the frame addresses and the corresponding content """
    frames_dict = dict()
    with open(args[1], 'r') as f:
        parse_frames = False
        for cnt, line in enumerate(f):
            if line.startswith("\n"):
                parse_frames = False
                continue
            line = line.strip()
            if not parse_frames and not line.startswith("Frame"):
                continue
            if line.startswith("Frame"):
                match = re.match("Frame 0x([0-9a-fA-F]+) ", line)
                frame_addr = "0x" + match.group(1).upper()
                parse_frames = True
                frames_dict[frame_addr] = list()
                continue
            for frame in line.split():
                frames_dict[frame_addr].append("0x" + frame.upper())
    for addr, words in frames_dict.items():
        print("{addr} {words}".format(addr=addr, words=",".join(words)))


if __name__ == "__main__":
    extract_frames(sys.argv)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
#
# Copyright (C) 2017-2020  The Project X-Ray Authors.
#
# Use of this source code is governed by a ISC-style
# license that can be found in the LICENSE file or at
# https://opensource.org/licenses/ISC
#
# SPDX-License-Identifier: ISC

from prjxray.segmaker import Segmaker
from prjxray import verilog
import json

# Set to true to enable additional tags useful for tracing bit toggles.
DEBUG_FUZZER = False


def bitfilter(frame, word):
    if frame < 25 or frame > 31:
        return False

    return True


def handle_data_width(segmk, d):
    if 'DATA_WIDTH' not in d:
        return

    site = d['ilogic_loc']

    # It appears several widths end up with the same bitstream pattern.
    # This groups those widths together for documentation.
    widths = [
        [2],
        [3],
        [4, 6],
        [5, 7],
        [8],
        [10],
        [14],
    ]

    width_map = {}

    for ws in widths:
        for w in ws:
            width_map[w] = 'W{}'.format('_'.join(map(str, ws)))

    zero_opt = 2
    W_OPT_ZERO = width_map[zero_opt]
    if d['DATA_WIDTH'] == zero_opt:
        segmk.add_site_tag(site, 'ISERDES.DATA_WIDTH.{}'.format(W_OPT_ZERO), 1)

        for opt in width_map.values():
            if opt == W_OPT_ZERO:
                continue

            segmk.add_site_tag(site, 'ISERDES.DATA_WIDTH.{}'.format(opt), 0)
    else:
        w_opt = width_map[d['DATA_WIDTH']]
        if w_opt != W_OPT_ZERO:
            segmk.add_site_tag(
                site, 'ISERDES.DATA_WIDTH.{}'.format(W_OPT_ZERO), 0)
            segmk.add_site_tag(site, 'ISERDES.DATA_WIDTH.{}'.format(w_opt), 1)


def handle_data_rate(segmk, d):
    if 'DATA_WIDTH' not in d:
        return

    site = d['ilogic_loc']

    for opt in ['SDR', 'DDR']:
        segmk.add_site_tag(
            site, 'ISERDES.DATA_RATE.{}'.format(opt),
            verilog.unquote(d['DATA_RATE']) == opt)


def main():
    print("Loading tags")
    segmk = Segmaker("design.bits")

    with open('params.jl', 'r') as f:
        design = json.load(f)

        for d in design:
            site = d['ilogic_loc']

            handle_data_width(segmk, d)
            handle_data_rate(segmk, d)

            segmk.add_site_tag(site, 'ISERDES.IN_USE', d['use_iserdese2'])

            if 'NUM_CE' in d:
                segmk.add_site_tag(site, 'ISERDES.NUM_CE.N2', d['NUM_CE'] == 2)

            segmk.add_site_tag(
                site, 'IDDR_OR_ISERDES.IN_USE', d['use_iserdese2']
                or d['iddr_mux_config'] != 'none')

            if 'INTERFACE_TYPE' in d:
                for opt in (
                        'MEMORY',
                        'MEMORY_DDR3',
                        'MEMORY_QDR',
                        'NETWORKING',
                        'OVERSAMPLE',
                ):
                    segmk.add_site_tag(
                        site, 'ISERDES.INTERFACE_TYPE.{}'.format(opt),
                        opt == verilog.unquote(d['INTERFACE_TYPE']))
                    segmk.add_site_tag(
                        site, 'ISERDES.INTERFACE_TYPE.Z_{}'.format(opt),
                        opt != verilog.unquote(d['INTERFACE_TYPE']))

                segmk.add_site_tag(
                    site, 'ISERDES.INTERFACE_TYPE.NOT_MEMORY',
                    'MEMORY' not in verilog.unquote(d['INTERFACE_TYPE']))

            if d['iddr_mux_config'] != 'none':
                segmk.add_site_tag(site, 'IFF.ZINIT_Q1', not d['INIT_Q1'])
                segmk.add_site_tag(site, 'IFF.ZINIT_Q2', not d['INIT_Q2'])

                if 'DYN_CLKDIV_INV_EN' in d:
                    segmk.add_site_tag(
                        site, 'DYN_CLKDIV_INV_EN',
                        verilog.unquote(d['DYN_CLKDIV_INV_EN']) == 'TRUE')

                if 'DYN_CLK_INV_EN' in d:
                    segmk.add_site_tag(
                        site, 'DYN_CLK_INV_EN',
                        verilog.unquote(d['DYN_CLK_INV_EN']) == 'TRUE')

                if 'INIT_Q3' in d:
                    segmk.add_site_tag(site, 'IFF.ZINIT_Q3', not d['INIT_Q3'])
                    segmk.add_site_tag(site, 'IFF.ZINIT_Q4', not d['INIT_Q4'])
                    segmk.add_site_tag(
                        site, 'IFF.ZSRVAL_Q1', not d['SRVAL_Q1'])
                    segmk.add_site_tag(
                        site, 'IFF.ZSRVAL_Q2', not d['SRVAL_Q2'])
                    segmk.add_site_tag(
                        site, 'IFF.ZSRVAL_Q3', not d['SRVAL_Q3'])
                    segmk.add_site_tag(
                        site, 'IFF.ZSRVAL_Q4', not d['SRVAL_Q4'])

                if 'IS_CLK_INVERTED' in d and not d['DISABLE_CLOCKS']:
                    if verilog.unquote(d['INTERFACE_TYPE']) == 'MEMORY_DDR3':
                        segmk.add_site_tag(
                            site, 'IFF.INV_CLK', d['IS_CLK_INVERTED'])
                        segmk.add_site_tag(
                            site, 'IFF.INV_CLKB', d['IS_CLKB_INVERTED'])
                        segmk.add_site_tag(
                            site, 'IFF.ZINV_CLK', not d['IS_CLK_INVERTED'])
                        segmk.add_site_tag(
                            site, 'IFF.ZINV_CLKB', not d['IS_CLKB_INVERTED'])

                        segmk.add_site_tag(
                            site, 'IFF.ZINV_CLK_XOR',
                            d['IS_CLK_INVERTED'] ^ d['IS_CLKB_INVERTED'])
                        segmk.add_site_tag(
                            site, 'IFF.ZINV_CLK_NXOR',
                            not (d['IS_CLK_INVERTED'] ^ d['IS_CLKB_INVERTED']))

                        segmk.add_site_tag(
                            site, 'IFF.ZINV_CLK_OR', d['IS_CLK_INVERTED']
                            or d['IS_CLKB_INVERTED'])
                        segmk.add_site_tag(
                            site, 'IFF.ZINV_CLK_NOR', not (
                                d['IS_CLK_INVERTED'] or d['IS_CLKB_INVERTED']))
                        segmk.add_site_tag(
                            site, 'IFF.ZINV_CLK_AND', d['IS_CLK_INVERTED']
                            and d['IS_CLKB_INVERTED'])
                        segmk.add_site_tag(
                            site, 'IFF.ZINV_CLK_NAND', not (
                                d['IS_CLK_INVERTED']
                                and d['IS_CLKB_INVERTED']))

                if 'IS_OCLK_INVERTED' in d and not d['DISABLE_CLOCKS']:
                    segmk.add_site_tag(
                        site, 'IFF.INV_OCLK', d['IS_OCLK_INVERTED'])
                    segmk.add_site_tag(
                        site, 'IFF.ZINV_OCLK', not d['IS_OCLK_INVERTED'])
                    segmk.add_site_tag(
                        site, 'IFF.INV_OCLKB', d['IS_OCLKB_INVERTED'])
                    segmk.add_site_tag(
                        site, 'IFF.ZINV_OCLKB', not d['IS_OCLKB_INVERTED'])

                if 'IS_CLKDIV_INVERTED' in d and not d['DISABLE_CLOCKS'] and \
                    verilog.unquote(d['INTERFACE_TYPE']) == 'MEMORY':
                    segmk.add_site_tag(
                        site, 'IFF.INV_CLKDIV', d['IS_CLKDIV_INVERTED'])
                    segmk.add_site_tag(
                        site, 'IFF.ZINV_CLKDIV', not d['IS_CLKDIV_INVERTED'])

                if 'IS_C_INVERTED' in d:
                    segmk.add_site_tag(
                        site, 'IFF.ZINV_C', not d['IS_C_INVERTED'])

                segmk.add_site_tag(site, 'ZINV_D', not d['IS_D_INVERTED'])

                if 'SRTYPE' in d:
                    for opt in ['ASYNC', 'SYNC']:
                        segmk.add_site_tag(
                            site, 'IFF.SRTYPE.{}'.format(opt),
                            verilog.unquote(d['SRTYPE']) == opt)

                if 'DDR_CLK_EDGE' in d:
                    for opt in ['OPPOSITE_EDGE', 'SAME_EDGE',
                                'SAME_EDGE_PIPELINED']:
                        segmk.add_site_tag(
                            site, 'IFF.DDR_CLK_EDGE.{}'.format(opt),
                            verilog.unquote(d['DDR_CLK_EDGE']) == opt)

            if d['iddr_mux_config'] == 'direct':
                segmk.add_site_tag(site, 'IFFDELMUXE3.P0', 0)
                segmk.add_site_tag(site, 'IFFDELMUXE3.P1', 1)
                segmk.add_site_tag(site, 'IFFDELMUXE3.P2', 0)
            elif d['iddr_mux_config'] == 'idelay':
                segmk.add_site_tag(site, 'IFFDELMUXE3.P0', 1)
                segmk.add_site_tag(site, 'IFFDELMUXE3.P1', 0)
                segmk.add_site_tag(site, 'IFFDELMUXE3.P2', 0)
            elif d['iddr_mux_config'] == 'none':
                segmk.add_site_tag(site, 'IFFDELMUXE3.P0', 0)
                segmk.add_site_tag(site, 'IFFDELMUXE3.P1', 0)
                segmk.add_site_tag(site, 'IFFDELMUXE3.P2', 0)
            else:
                assert False, d['mux_config']

            if d['mux_config'] == 'direct':
                segmk.add_site_tag(site, 'IDELMUXE3.P0', 0)
                segmk.add_site_tag(site, 'IDELMUXE3.P1', 1)
                segmk.add_site_tag(site, 'IDELMUXE3.P2', 0)

            elif d['mux_config'] == 'idelay':
                segmk.add_site_tag(site, 'IDELMUXE3.P0', 1)
                segmk.add_site_tag(site, 'IDELMUXE3.P1', 0)
                segmk.add_site_tag(site, 'IDELMUXE3.P2', 0)

            elif d['mux_config'] == 'none':
                segmk.add_site_tag(site, 'IDELMUXE3.P0', 0)
                segmk.add_site_tag(site, 'IDELMUXE3.P1', 0)
                segmk.add_site_tag(site, 'IDELMUXE3.P2', 0)
            else:
                assert False, d['mux_config']

            if DEBUG_FUZZER:
                for k in d:
                    segmk.add_site_tag(
                        site, 'param_' + k + '_' + str(d[k]).replace(
                            ' ', '').replace('\n', ''), 1)

    segmk.compile(bitfilter=bitfilter)
    segmk.write(allow_empty=True)


if __name__ == "__main__":
    main()



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
#
# Copyright (C) 2017-2020  The Project X-Ray Authors.
#
# Use of this source code is governed by a ISC-style
# license that can be found in the LICENSE file or at
# https://opensource.org/licenses/ISC
#
# SPDX-License-Identifier: ISC

import json
import csv

from prjxray.segmaker import Segmaker
from prjxray import verilog
from prjxray import segmaker


def isinv_tags(segmk, ps, site, actual_ps):
    # all of these bits are inverted
    ks = [
        ('IS_CLKARDCLK_INVERTED', 'ZINV_CLKARDCLK'),
        ('IS_CLKBWRCLK_INVERTED', 'ZINV_CLKBWRCLK'),
        ('IS_REGCLKARDRCLK_INVERTED', 'ZINV_REGCLKARDRCLK'),
        ('IS_REGCLKB_INVERTED', 'ZINV_REGCLKB'),
        ('IS_ENARDEN_INVERTED', 'ZINV_ENARDEN'),
        ('IS_ENBWREN_INVERTED', 'ZINV_ENBWREN'),
        ('IS_RSTRAMARSTRAM_INVERTED', 'ZINV_RSTRAMARSTRAM'),
        ('IS_RSTRAMB_INVERTED', 'ZINV_RSTRAMB'),
        ('IS_RSTREGARSTREG_INVERTED', 'ZINV_RSTREGARSTREG'),
        ('IS_RSTREGB_INVERTED', 'ZINV_RSTREGB'),
    ]

    for param, tagname in ks:
        # The CLK inverts sometimes are changed during synthesis, resulting
        # in addition inversions.  Take this into account.
        if param in actual_ps:
            tag = 1 ^ verilog.parsei(actual_ps[param])
        elif param == 'IS_REGCLKARDRCLK_INVERTED':
            if verilog.parsei(ps['DOA_REG']):
                # When DOA_REG == 1, REGCLKARDRCLK follows the CLKARDCLK setting.
                tag = 1 ^ verilog.parsei(actual_ps['IS_CLKARDCLK_INVERTED'])
            else:
                # When DOA_REG == 0, REGCLKARDRCLK is always inverted.
                tag = 0

            segmk.add_site_tag(site, tagname, tag)
        elif param == 'IS_REGCLKB_INVERTED':
            if verilog.parsei(ps['DOB_REG']):
                # When DOB_REG == 1, REGCLKB follows the CLKBWRCLK setting.
                tag = 1 ^ verilog.parsei(actual_ps['IS_CLKBWRCLK_INVERTED'])
            else:
                # When DOB_REG == 0, REGCLKB is always inverted.
                tag = 0

        else:
            tag = 1 ^ verilog.parsei(ps[param])

        segmk.add_site_tag(site, tagname, tag)


def bus_tags(segmk, ps, site):
    for param in ("DOA_REG", "DOB_REG"):
        segmk.add_site_tag(site, param, verilog.parsei(ps[param]))

    for param, tagname in [('SRVAL_A', 'ZSRVAL_A'), ('SRVAL_B', 'ZSRVAL_B'),
                           ('INIT_A', 'ZINIT_A'), ('INIT_B', 'ZINIT_B')]:
        bitstr = verilog.parse_bitstr(ps[param])
        ab = param[-1]
        # Are all bits present?
        hasparity = ps['READ_WIDTH_' + ab] == 18
        for i in range(18):
            # Magic bit positions from experimentation
            # we could just only solve when parity, but this check documents the fine points a bit better
            if hasparity or i not in (1, 9):
                segmk.add_site_tag(
                    site, '%s[%u]' % (tagname, i), 1 ^ bitstr[i])


def rw_width_tags(segmk, ps, site):
    '''
    Y0.READ_WIDTH_A
    width   001_03  001_04  001_05
    1       0       0       0
    2       1       0       0
    4       0       1       0
    9       1       1       0
    18      0       0       1
    '''
    params = ["READ_WIDTH_A", "READ_WIDTH_B", "WRITE_WIDTH_A", "WRITE_WIDTH_B"]

    for param in params:
        set_val = int(ps[param])

        if set_val == 0:
            set_val = 1

        if set_val >= 36:
            continue

        def mk(x):
            return '%s_%u' % (param, x)

        segmaker.add_site_group_zero(
            segmk, site, "",
            [mk(1), mk(2), mk(4), mk(9), mk(18)], mk(1), mk(set_val))


def write_mode_tags(segmk, ps, site):
    for param in ["WRITE_MODE_A", "WRITE_MODE_B"]:
        set_val = verilog.unquote(ps[param])
        # WRITE_FIRST: no bits set
        segmk.add_site_tag(
            site, '%s_READ_FIRST' % (param), set_val == "READ_FIRST")
        segmk.add_site_tag(
            site, '%s_NO_CHANGE' % (param), set_val == "NO_CHANGE")


def write_rstreg_priority(segmk, ps, site):
    for param in ["RSTREG_PRIORITY_A", "RSTREG_PRIORITY_B"]:
        set_val = verilog.unquote(ps[param])
        for opt in ["RSTREG", "REGCE"]:
            segmk.add_site_tag(
                site, "{}_{}".format(param, opt), set_val == opt)


def write_rdaddr_collision(segmk, ps, site):
    for opt in ["DELAYED_WRITE", "PERFORMANCE"]:
        set_val = verilog.unquote(ps['RDADDR_COLLISION_HWCONFIG'])
        segmk.add_site_tag(
            site, "RDADDR_COLLISION_HWCONFIG_{}".format(opt), set_val == opt)


def run():

    segmk = Segmaker("design.bits")

    clk_inverts = {}
    with open('design.csv', 'r') as f:
        for params in csv.DictReader(f):
            clk_inverts[params['site']] = params

    print("Loading tags")
    f = open('params.jl', 'r')
    f.readline()
    for l in f:
        j = json.loads(l)
        ps = j['params']
        assert j['module'] == 'my_RAMB18E1'
        site = verilog.unquote(ps['LOC'])

        bus_tags(segmk, ps, site)
        if ps['RAM_MODE'] == '"TDP"':
            rw_width_tags(segmk, ps, site)
        segmk.add_site_tag(
            site, 'SDP_READ_WIDTH_36', ps['RAM_MODE'] == '"SDP"'
            and int(ps['READ_WIDTH_A']) == 36)
        segmk.add_site_tag(
            site, 'SDP_WRITE_WIDTH_36', ps['RAM_MODE'] == '"SDP"'
            and int(ps['WRITE_WIDTH_B']) == 36)

        if ps['READ_WIDTH_A'] < 36 and ps['WRITE_WIDTH_B'] < 36:
            isinv_tags(segmk, ps, site, clk_inverts[site])
            write_mode_tags(segmk, ps, site)
            write_rstreg_priority(segmk, ps, site)
            write_rdaddr_collision(segmk, ps, site)

    def bitfilter(frame, bit):
        # rw_width_tags() aliasing interconnect on large widths
        return frame not in (0, 20, 21)

    segmk.compile(bitfilter=bitfilter)
    segmk.write()


run()



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
#
# Copyright (C) 2017-2020  The Project X-Ray Authors.
#
# Use of this source code is governed by a ISC-style
# license that can be found in the LICENSE file or at
# https://opensource.org/licenses/ISC
#
# SPDX-License-Identifier: ISC
'''
Need coverage for the following:
RAM32X1S_N
RAM32X1D
RAM32M
RAM64X1S_N
RAM64X1D_N
RAM64M
RAM128X1S_N
RAM128X1D
RAM256X1S
SRL16E_N
SRLC32E_N

Note: LUT6 was added to try to simplify reduction, although it might not be needed
'''

import os, random
random.seed(0)
from prjxray import util
from prjxray import verilog

# INCREMENT is the amount of additional CLBN to be instantiated in the design.
# This makes the fuzzer compilation more robust against failures.
INCREMENT = os.getenv('CLBN', 0)
CLBN = 50 + int(INCREMENT)
print('//Requested CLBs: %s' % str(CLBN))


def gen_slicems():
    for _tile_name, site_name, _site_type in util.get_roi().gen_sites(
        ['SLICEM']):
        yield site_name


DIN_N = CLBN * 8
DOUT_N = CLBN * 8

verilog.top_harness(DIN_N, DOUT_N)

f = open('params.csv', 'w')
f.write('module,loc,bela,belb,belc,beld\n')
slices = sorted(gen_slicems())
random.shuffle(slices)

print(
    'module roi(input clk, input [%d:0] din, output [%d:0] dout);' %
    (DIN_N - 1, DOUT_N - 1))
randluts = 0
for clbi, loc in zip(range(CLBN), slices):
    params = ''
    cparams = ''
    # Multi module
    # Fill with random assortment of SRL16E and SRLC32E
    if random.randint(0, 1):
        params = ''
        module = 'my_ram_N'

        # Can fit 4 per CLB
        # BELable
        bel_opts = [
            'SRL16E',
            'SRLC32E',
            # Weight LUT6 more heavily to make WEMUX.CE solve quicker
            'LUT6',
            'LUT6',
        ]

        bels = []
        for beli in range(4):
            belc = chr(ord('A') + beli)
            if randluts:
                bel = random.choice(bel_opts)
            else:
                # Force one without memory elements to bring CE bit low
                bel = 'LUT6'

            params += ', .%c_%s(1)' % (belc, bel)
            bels.append(bel)
        # Record the BELs we chose in the module (A, B, C, D)
        cparams = ',' + (','.join(bels))
        randluts += 1
    # Greedy module
    # Don't place anything else in it
    # For solving muxes vs previous results
    else:
        modules = [
            # (module,          N max, FF opt)
            ('my_RAM32X1S_N', 4, 0),
            ('my_RAM32X1D', None, 0),
            ('my_RAM32M', None, 0),
            ('my_RAM64X1S_N', 4, 0),
            ('my_RAM64X1D_N', 2, 0),
            ('my_RAM64M', None, 0),
            ('my_RAM128X1S_N', 2, 1),
            ('my_RAM128X1D', None, 1),
            ('my_RAM256X1S', None, 1),
        ]

        module, nmax, ff_param = random.choice(modules)

        n = ''
        if nmax:
            n = random.randint(1, nmax)
            params += ',.N(%d)' % n
        cparams += ',%s' % n

        ff = ''
        if ff_param:
            ff = random.randint(0, 1)
            params += ',.FF(%d)' % ff
        cparams += ',%s' % ff

        # Pad to match above CSV size
        cparams += ",,"

    print('    %s' % module)
    print('            #(.LOC("%s")%s)' % (loc, params))
    sel = random.random()
    if sel > .15:
        print(
            '            clb_%d (.clk(clk), .din(din[  %d +: 8]), .dout(dout[  %d +: 8]));'
            % (clbi, 8 * clbi, 8 * clbi))
    else:
        print(
            "            clb_%d (.clk(clk), .din({8{1'b1}}), .dout());" %
            (clbi, ))

    f.write('%s,%s%s\n' % (module, loc, cparams))
f.close()
print(
    '''endmodule

// ---------------------------------------------------------------------

''')

print(
    '''

//***************************************************************
//Basic

module maybe_ff (input clk, din, dout);
    parameter FF = 0;

    generate
        if (FF) begin
            reg r;
            assign dout = r;
            always @(posedge clk) begin
                r = din;
            end
        end else begin
            assign dout = din;
        end
    endgenerate
endmodule

module my_RAM32X1S_N (input clk, input [7:0] din, output [7:0] dout);
    parameter LOC = "";
    //1-4
    parameter N=1;

    genvar i;
    generate
        for (i = 0; i < N; i = i + 1) begin : loop
            (* LOC=LOC, KEEP, DONT_TOUCH *)
            RAM32X1S #(
                ) RAM32X1S (
                    .O(dout[i]),
                    .A0(din[0]),
                    .A1(din[1]),
                    .A2(din[2]),
                    .A3(din[3]),
                    .A4(din[4]),
                    .D(din[5]),
                    .WCLK(clk),
                    .WE(ce));
        end
    endgenerate
endmodule

module my_RAM32X1D (input clk, input [7:0] din, output [7:0] dout);
    parameter LOC = "";

    (* LOC=LOC, KEEP, DONT_TOUCH *)
    RAM32X1D #(
        ) RAM32X1D (
            .DPO(dout[0]),
            .SPO(dout[1]),
            .A0(din[0]),
            .A1(din[1]),
            .A2(din[2]),
            .A3(din[3]),
            .A4(din[4]),
            .D(din[5]),
            .DPRA0(din[6]),
            .DPRA1(din[7]),
            .DPRA2(din[0]),
            .DPRA3(din[1]),
            .DPRA4(din[2]),
            .WCLK(din[3]),
            .WE(din[4]));
endmodule

module my_RAM32M (input clk, input [7:0] din, output [7:0] dout);
    parameter LOC = "";

    (* LOC=LOC, KEEP, DONT_TOUCH *)
    RAM32M #(
        ) RAM32M (
            .DOA(dout[1:0]),
            .DOB(dout[3:2]),
            .DOC(dout[5:4]),
            .DOD(dout[7:6]),
            .ADDRA(din[4:0]),
            .ADDRB(din[4:0]),
            .ADDRC(din[4:0]),
            .ADDRD(din[4:0]),
            .DIA(din[5:4]),
            .DIB(din[6:5]),
            .DIC(din[7:6]),
            .DID(din[1:0]),
            .WCLK(din[1]),
            .WE(din[2]));
endmodule

module my_RAM64X1S_N (input clk, input [7:0] din, output [7:0] dout);
    parameter LOC = "";
    //1-4
    parameter N=1;

    genvar i;
    generate
        for (i = 0; i < N; i = i + 1) begin : loop
            (* LOC=LOC, KEEP, DONT_TOUCH *)
            RAM64X1S #(
                ) RAM64X1S (
                    .O(dout[i]),
                    .A0(din[0]),
                    .A1(din[1]),
                    .A2(din[2]),
                    .A3(din[3]),
                    .A4(din[4]),
                    .A5(din[5]),
                    .D(din[6]),
                    .WCLK(clk),
                    .WE(ce));
        end
    endgenerate
endmodule

module my_RAM64X1D_N (input clk, input [7:0] din, output [7:0] dout);
    parameter LOC = "";
    //1-2
    parameter N=1;

    genvar i;
    generate
        for (i = 0; i < N; i = i + 1) begin : loop
            (* LOC=LOC, KEEP, DONT_TOUCH *)
            RAM64X1D #(
                    .INIT(64'h0),
                    .IS_WCLK_INVERTED(1'b0)
                ) ramb (
                    .DPO(dout[i]),
                    .D(din[0]),
                    .WCLK(clk),
                    .WE(din[2]),
                    .A0(din[3]),
                    .A1(din[4]),
                    .A2(din[5]),
                    .A3(din[6]),
                    .A4(din[7]),
                    .A5(din[0]),
                    .DPRA0(din[1]),
                    .DPRA1(din[2]),
                    .DPRA2(din[3]),
                    .DPRA3(din[4]),
                    .DPRA4(din[5]),
                    .DPRA5(din[6]));
        end
    endgenerate
endmodule

module my_RAM64M (input clk, input [7:0] din, output [7:0] dout);
    parameter LOC = "";
    parameter BEL="A6LUT";

    (* LOC=LOC, BEL=BEL, KEEP, DONT_TOUCH *)
    RAM64M #(
        ) RAM64M (
            .DOA(dout[0]),
            .DOB(dout[1]),
            .DOC(dout[2]),
            .DOD(dout[3]),
            .ADDRA(din[0]),
            .ADDRB(din[1]),
            .ADDRC(din[2]),
            .ADDRD(din[3]),
            .DIA(din[4]),
            .DIB(din[5]),
            .DIC(din[6]),
            .DID(din[7]),
            .WCLK(clk),
            .WE(din[1]));
endmodule

module my_RAM128X1S_N (input clk, input [7:0] din, output [7:0] dout);
    parameter LOC = "";
    //1-2
    parameter N=1;
    parameter FF = 0;

    genvar i;
    generate
        for (i = 0; i < N; i = i + 1) begin : loop
            wire o;

            (* LOC=LOC, KEEP, DONT_TOUCH *)
            RAM128X1S #(
                ) RAM128X1S (
                    .O(o),
                    .A0(din[0]),
                    .A1(din[1]),
                    .A2(din[2]),
                    .A3(din[3]),
                    .A4(din[4]),
                    .A5(din[5]),
                    .A6(din[6]),
                    .D(din[7]),
                    .WCLK(din[0]),
                    .WE(din[1]));

            maybe_ff #(.FF(FF)) ff (.clk(clk), .din(o), .dout(dout[i]));
        end
    endgenerate
endmodule

module my_RAM128X1D (input clk, input [7:0] din, output [7:0] dout);
    parameter LOC = "";
    parameter FF = 0;

    wire dpo, spo;

    (* LOC=LOC, KEEP, DONT_TOUCH *)
    RAM128X1D #(
            .INIT(128'h0),
            .IS_WCLK_INVERTED(1'b0)
        ) RAM128X1D (
            .DPO(dpo),
            .SPO(spo),
            .D(din[0]),
            .WCLK(clk),
            .WE(din[2]));

    maybe_ff #(.FF(FF)) ff0 (.clk(clk), .din(dpo), .dout(dout[0]));
    maybe_ff #(.FF(FF)) ff1 (.clk(clk), .din(spo), .dout(dout[1]));
endmodule

//Dedicated LOC
module my_RAM256X1S (input clk, input [7:0] din, output [7:0] dout);
    parameter LOC = "";
    parameter FF = 0;

    wire o;

    (* LOC=LOC, KEEP, DONT_TOUCH *)
    RAM256X1S #(
        ) RAM256X1S (
            .O(o),
            .A({din[0], din[7:0]}),
            .D(din[0]),
            .WCLK(din[1]),
            .WE(din[2]));

    maybe_ff #(.FF(FF)) ff (.clk(clk), .din(o), .dout(dout[0]));
endmodule

//***************************************************************

module my_ram_N_inst (input clk, input [7:0] din, output dout);
    parameter LOC="";
    parameter BEL="";
    parameter N_SRL16E=0;
    parameter N_SRLC32E=0;
    parameter N_LUT6=0;

    parameter SRLINIT = 32'h00000000;
    parameter LUTINIT6 = 64'h0000_0000_0000_0000;

    wire ce = din[4];

    generate
        //********************
        if (N_SRL16E) begin
            (* LOC=LOC, BEL=BEL, KEEP, DONT_TOUCH *)
            SRL16E #(
                ) lut (
                    .Q(dout),
                    .A0(din[0]),
                    .A1(din[1]),
                    .A2(din[2]),
                    .A3(din[3]),
                    .CE(ce),
                    .CLK(clk),
                    .D(din[6]));
        end
        if (N_SRLC32E) begin
            (* LOC=LOC, BEL=BEL, KEEP, DONT_TOUCH *)
            SRLC32E #(
                    .INIT(SRLINIT),
                    .IS_CLK_INVERTED(1'b0)
                ) lut (
                    .Q(dout),
                    .Q31(),
                    .A(din[4:0]),
                    .CE(ce),
                    .CLK(clk),
                    .D(din[7]));
        end
        if (N_LUT6) begin
	        (* LOC=LOC, BEL=BEL, KEEP, DONT_TOUCH *)
	        LUT6_2 #(
		        .INIT(LUTINIT6)
	        ) lut (
		        .I0(din[0]),
		        .I1(din[1]),
		        .I2(din[2]),
		        .I3(din[3]),
		        .I4(din[4]),
		        .I5(din[5]),
		        .O5(),
		        .O6(dout));
        end
    endgenerate
endmodule

/*
Supermodule for LOC'able prims
Mix and match as needed
Specify at most one function generator per LUT
*/
module my_ram_N (input clk, input [7:0] din, output [7:0] dout);
    parameter LOC = "";
    parameter D_SRL16E=0;
    parameter D_SRLC32E=0;
    parameter D_LUT6=0;

    parameter C_SRL16E=0;
    parameter C_SRLC32E=0;
    parameter C_LUT6=0;

    parameter B_SRL16E=0;
    parameter B_SRLC32E=0;
    parameter B_LUT6=0;

    parameter A_SRL16E=0;
    parameter A_SRLC32E=0;
    parameter A_LUT6=0;

    my_ram_N_inst #(.LOC(LOC), .BEL("D6LUT"), .N_SRL16E(D_SRL16E), .N_SRLC32E(D_SRLC32E), .N_LUT6(D_LUT6))
            lutd(.clk(clk), .din(din), .dout(dout[3]));
    my_ram_N_inst #(.LOC(LOC), .BEL("C6LUT"), .N_SRL16E(C_SRL16E), .N_SRLC32E(C_SRLC32E), .N_LUT6(C_LUT6))
            lutc(.clk(clk), .din(din), .dout(dout[2]));
    my_ram_N_inst #(.LOC(LOC), .BEL("B6LUT"), .N_SRL16E(B_SRL16E), .N_SRLC32E(B_SRLC32E), .N_LUT6(B_LUT6))
            lutb(.clk(clk), .din(din), .dout(dout[1]));
    my_ram_N_inst #(.LOC(LOC), .BEL("A6LUT"), .N_SRL16E(A_SRL16E), .N_SRLC32E(A_SRLC32E), .N_LUT6(A_LUT6))
            luta(.clk(clk), .din(din), .dout(dout[0]));
endmodule

''')



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
#
# Copyright (C) 2017-2020  The Project X-Ray Authors.
#
# Use of this source code is governed by a ISC-style
# license that can be found in the LICENSE file or at
# https://opensource.org/licenses/ISC
#
# SPDX-License-Identifier: ISC
import json
import io
import os
import random
import re
random.seed(int(os.getenv("SEED"), 16))
from prjxray import util
from prjxray import lut_maker
from prjxray import verilog
from prjxray.db import Database


def gen_sites():
    db = Database(util.get_db_root(), util.get_part())
    grid = db.grid()
    for tile_name in sorted(grid.tiles()):
        loc = grid.loc_of_tilename(tile_name)
        gridinfo = grid.gridinfo_at_loc(loc)
        if gridinfo.tile_type.endswith("_SING"):
            continue
        # Y9 tiles have frame address 1 frame higher than the rest
        # Need to investigate what is so special about them
        if tile_name.endswith("Y9"):
            continue

        sites = []
        for site_name, site_type in gridinfo.sites.items():
            if site_type == 'IDELAYE2_FINEDELAY':
                yield tile_name, site_name


def write_params(params):
    pinstr = 'tile,isone,site\n'
    for vals in params:
        pinstr += ','.join(map(str, vals)) + '\n'

    open('params.csv', 'w').write(pinstr)


def use_idelay(p, luts, connects):
    print(
        '''
    wire idelay_{site};

    (* KEEP, DONT_TOUCH, LOC = "{site}" *)
    IDELAYE2_FINEDELAY #(
        .HIGH_PERFORMANCE_MODE("{param}"),
        .DELAY_SRC("DATAIN")
    ) idelay_site_{site} (
        .DATAIN({onet}),
        .DATAOUT(idelay_{site})
        );
    assign {net} = idelay_{site};
       '''.format(
            onet=luts.get_next_output_net(),
            net=luts.get_next_input_net(),
            param="TRUE" if p['isone'] else "FALSE",
            **p),
        file=connects)


def run():
    luts = lut_maker.LutMaker()
    connects = io.StringIO()

    tile_params = []
    params = []
    sites = sorted(list(gen_sites()))
    for idx, ((tile, site), isone) in enumerate(zip(
            sites, util.gen_fuzz_states(len(sites)))):

        p = {}
        p['tile'] = tile
        p['site'] = site
        p['isone'] = isone
        params.append(p)
        tile_params.append((tile, p['isone'], site))

    write_params(tile_params)

    print('''
module top();
    ''')

    # Always output a LUT6 to make placer happy.
    print('''
     (* KEEP, DONT_TOUCH *)
     LUT6 dummy_lut();
     ''')

    # Need IDELAYCTRL for IDEALAYs
    print('''
     (* KEEP, DONT_TOUCH *)
     IDELAYCTRL();
     ''')

    for p in params:
        use_idelay(p, luts, connects)

    for l in luts.create_wires_and_luts():
        print(l)

    print(connects.getvalue())

    print("endmodule")


if __name__ == '__main__':
    run()



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
#
# Copyright (C) 2017-2020  The Project X-Ray Authors.
#
# Use of this source code is governed by a ISC-style
# license that can be found in the LICENSE file or at
# https://opensource.org/licenses/ISC
#
# SPDX-License-Identifier: ISC
class Overlay(object):
    """ Object that represents an overlay.

    Can be used to iterate over tiles and sites not inside a partition region.

    """

    def __init__(self, region_dict):
        self.region_dict = region_dict

    def tile_in_roi(self, grid_loc):
        """ Returns true if grid_loc (GridLoc tuple) is within the overlay. """
        x = grid_loc.grid_x
        y = grid_loc.grid_y
        for _, bounds in self.region_dict.items():
            x1, x2, y1, y2 = bounds
            if x1 <= x and x <= x2 and y1 <= y and y <= y2:
                return False
        return True



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
#
# Copyright (C) 2017-2020  The Project X-Ray Authors.
#
# Use of this source code is governed by a ISC-style
# license that can be found in the LICENSE file or at
# https://opensource.org/licenses/ISC
#
# SPDX-License-Identifier: ISC

import subprocess
import demo_sw_led_fasm
import os


def run(roi_dir, swn, ledn):
    design_txt_fn = roi_dir + '/design.txt'
    bit_ref_fn = roi_dir + '/design.bit'
    fasm_fn = 'demo_sw_led.fasm'
    bit_out_fn = 'demo_sw_led.bit'
    ocd_cfg = os.getenv(
        'XRAY_DIR') + '/utils/openocd/board-digilent-basys3.cfg'

    # Clean up old tmp files to be sure we are generating them fresh
    subprocess.call('rm -f %s %s' % (fasm_fn, bit_out_fn), shell=True)

    # subprocess.shell("python3 demo_sw_led.py out_xc7a35tcpg236-1_BASYS3-SWBUT_roi_basev/design.txt 0 0 demo.fasm")
    demo_sw_led_fasm.run(
        open(design_txt_fn, 'r'), swn, ledn, open(fasm_fn, 'w'))
    subprocess.check_call(
        "./fasm2bit.sh %s %s %s" % (fasm_fn, bit_ref_fn, bit_out_fn),
        shell=True)
    subprocess.check_call(
        'openocd -f %s -c "init; pld load 0 %s; exit"' % (ocd_cfg, bit_out_fn),
        shell=True)


if __name__ == '__main__':
    import argparse

    parser = argparse.ArgumentParser(
        description=
        'Basys3 switch to LED interconnect demo. Compiles and programs')
    parser.add_argument(
        'roi_dir', help='ROI project dir for harness .bit and  metadata.txt')
    parser.add_argument('sw', type=int, help='Switch to use')
    parser.add_argument('led', type=int, help='LED to use')

    args = parser.parse_args()
    run(args.roi_dir, args.sw, args.led)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
#
# Copyright (C) 2017-2020  The Project X-Ray Authors.
#
# Use of this source code is governed by a ISC-style
# license that can be found in the LICENSE file or at
# https://opensource.org/licenses/ISC
#
# SPDX-License-Identifier: ISC
import os
import shutil
import sys
import subprocess
import signal
from multiprocessing import Pool, Lock
from itertools import chain
import argparse

# Can be used to redirect vivado tons of output
# stdout=DEVNULL in subprocess.check_call


# Worker function called from threads.
# Once the worker completes the job, the temporary files
# get merged with the final outputs in a thread-safe way
# and deleted to save disk usage.
# To do so, a global Lock is provided at the Pool initialization.
def start_pips(argList):
    blockID, start, stop, total = argList
    print("Running instance :" + str(blockID) + " / " + str(total))
    subprocess.check_call(
        "${XRAY_VIVADO} -mode batch -source $FUZDIR/job.tcl -tclargs " +
        str(blockID) + " " + str(start) + " " + str(stop),
        shell=True)

    uphill_wires = "wires/uphill_wires_{}.txt".format(blockID)
    downhill_wires = "wires/downhill_wires_{}.txt".format(blockID)

    # Locking to write on final file and remove the temporary one
    Lock.acquire()
    with open("uphill_wires.txt", "a") as wfd:
        f = uphill_wires
        with open(f, "r") as fd:
            shutil.copyfileobj(fd, wfd)

    with open("downhill_wires.txt", "a") as wfd:
        f = downhill_wires
        with open(f, "r") as fd:
            shutil.copyfileobj(fd, wfd)
    Lock.release()

    os.remove(uphill_wires)
    os.remove(downhill_wires)


# Function called once to get the total numbers of pips to list
def get_nb_pips():
    print("Fetching total number of pips")
    subprocess.check_call(
        "${XRAY_VIVADO} -mode batch -source $FUZDIR/get_pipscount.tcl",
        shell=True)
    countfile = open("nb_pips.txt", "r")
    return int(countfile.readline())


def pool_init(lock):
    global Lock
    Lock = lock


def run_pool(itemcount, nbBlocks, blocksize, nbParBlock, workFunc):
    # We handle the case of not integer multiple of pips
    intitemcount = blocksize * nbBlocks
    lastRun = False
    modBlocks = itemcount - intitemcount
    if modBlocks != 0:
        lastRun = True
        nbBlocks = nbBlocks + 1

    print(
        "Items Count: " + str(itemcount) + " - Number of blocks: " +
        str(nbBlocks) + " - Parallel blocks: " + str(nbParBlock) +
        " - Blocksize: " + str(blocksize) + " - Modulo Blocks: " +
        str(modBlocks))

    blockId = range(0, nbBlocks)
    startI = range(0, intitemcount, blocksize)
    stopI = range(blocksize, intitemcount + 1, blocksize)
    totalBlock = [nbBlocks for _ in range(nbBlocks)]

    # In case we have a last incomplete block we add it as a last
    # element in the arguments list
    if lastRun == True:
        startI = chain(startI, [intitemcount])
        stopI = chain(stopI, [itemcount])

    mpLock = Lock()

    argList = zip(blockId, startI, stopI, totalBlock)

    with Pool(processes=nbParBlock, initializer=pool_init,
              initargs=(mpLock, )) as pool:
        pool.map(workFunc, argList)

    return nbBlocks


# ==========================================================================
# ===== FPGA Logic Items data ==============================================
# For Artix 7 50T:
#   - Total pips: 22002368
#   - Total tiles: 18055
#   - Total nodes: 1953452
# For Kintex 7 70T:
#   - Total pips: 29424910
#   - Total tiles: 24453
#   - Total nodes: 2663055
# For Zynq 7 z010:
#   - Total pips: 12462138
#   - Total tiles: 13440
#   - Total nodes: 1122477
# =========================================================================
# Dividing by about 64 over 4 core is not optimized but a default to run
# on most computer
# =========================================================================


def main(argv):
    parser = argparse.ArgumentParser()
    parser.add_argument(
        "-p",
        "--nbPar",
        help="Number of parallel instances of Vivado",
        type=int,
        default=4)
    parser.add_argument(
        "-t",
        "--sizePipsBlock",
        help="Define the number of pips to process per instance",
        type=int,
        default=340000)
    args = parser.parse_args()

    nbParBlock = args.nbPar
    blockPipsSize = args.sizePipsBlock

    pipscount = get_nb_pips()
    nbPipsBlock = int(pipscount / blockPipsSize)

    if not os.path.exists("wires"):
        os.mkdir("wires")

    print(
        " nbPar: " + str(nbParBlock) + " blockPipsSize: " + str(blockPipsSize))

    pipsFileCount = run_pool(
        pipscount, nbPipsBlock, blockPipsSize, nbParBlock, start_pips)

    print("Work done !")
    return 0


if __name__ == "__main__":
    sys.exit(main(sys.argv))



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
#
# Copyright (C) 2017-2020  The Project X-Ray Authors.
#
# Use of this source code is governed by a ISC-style
# license that can be found in the LICENSE file or at
# https://opensource.org/licenses/ISC
#
# SPDX-License-Identifier: ISC
import os
import random
random.seed(int(os.getenv("SEED"), 16))
from prjxray import util


def write_params(params):
    pinstr = 'tile,val\n'
    for tile, (val) in sorted(params.items()):
        pinstr += '%s,%s\n' % (tile, val)
    open('params.csv', 'w').write(pinstr)


def run():
    print(
        '''
module top(input clk, stb, di, output do);
    localparam integer DIN_N = 8;
    localparam integer DOUT_N = 8;

    reg [DIN_N-1:0] din;
    wire [DOUT_N-1:0] dout;

    reg [DIN_N-1:0] din_shr;
    reg [DOUT_N-1:0] dout_shr;

    always @(posedge clk) begin
        din_shr <= {din_shr, di};
        dout_shr <= {dout_shr, din_shr[DIN_N-1]};
        if (stb) begin
            din <= din_shr;
            dout_shr <= dout;
        end
    end

    assign do = dout_shr[DOUT_N-1];
    ''')

    params = {}

    # NOTE: The INT_L tile has been hardcoded and it works only for the part specified in the assertion
    database = os.getenv('XRAY_DATABASE')
    int_tile = os.getenv('XRAY_PS7_INT')
    assert database == "zynq7", database
    for isone in util.gen_fuzz_states(1):
        params[int_tile] = isone
        print(
            '''
    (* KEEP, DONT_TOUCH *)

    PS7 dut_%(dut)s(
	.DMA0DATYPE			(),
	.DMA0DAVALID			(),
	.DMA0DRREADY			(),
	.DMA0RSTN			(),
	.DMA1DATYPE	              	(),
	.DMA1DAVALID			(),
	.DMA1DRREADY			(),
	.DMA1RSTN			(),
	.DMA2DATYPE			(),
	.DMA2DAVALID			(),
	.DMA2DRREADY			(),
	.DMA2RSTN			(),
	.DMA3DATYPE			(),
	.DMA3DAVALID			(),
	.DMA3DRREADY			(),
	.DMA3RSTN			(),
	.EMIOCAN0PHYTX			(),
	.EMIOCAN1PHYTX			(),
	.EMIOENET0GMIITXD		(),
	.EMIOENET0GMIITXEN		(),
	.EMIOENET0GMIITXER		(),
	.EMIOENET0MDIOMDC		(),
	.EMIOENET0MDIOO			(),
	.EMIOENET0MDIOTN		(),
	.EMIOENET0PTPDELAYREQRX		(),
	.EMIOENET0PTPDELAYREQTX		(),
	.EMIOENET0PTPPDELAYREQRX	(),
	.EMIOENET0PTPPDELAYREQTX	(),
	.EMIOENET0PTPPDELAYRESPRX	(),
	.EMIOENET0PTPPDELAYRESPTX	(),
	.EMIOENET0PTPSYNCFRAMERX	(),
	.EMIOENET0PTPSYNCFRAMETX	(),
	.EMIOENET0SOFRX			(),
	.EMIOENET0SOFTX			(),
	.EMIOENET1GMIITXD		(),
	.EMIOENET1GMIITXEN		(),
	.EMIOENET1GMIITXER		(),
	.EMIOENET1MDIOMDC		(),
	.EMIOENET1MDIOO			(),
	.EMIOENET1MDIOTN		(),
	.EMIOENET1PTPDELAYREQRX		(),
	.EMIOENET1PTPDELAYREQTX		(),
	.EMIOENET1PTPPDELAYREQRX	(),
	.EMIOENET1PTPPDELAYREQTX	(),
	.EMIOENET1PTPPDELAYRESPRX	(),
	.EMIOENET1PTPPDELAYRESPTX	(),
	.EMIOENET1PTPSYNCFRAMERX	(),
	.EMIOENET1PTPSYNCFRAMETX	(),
	.EMIOENET1SOFRX			(),
	.EMIOENET1SOFTX			(),
	.EMIOGPIOO			(),
	.EMIOGPIOTN			(),
	.EMIOI2C0SCLO			(),
	.EMIOI2C0SCLTN			(),
	.EMIOI2C0SDAO			(),
	.EMIOI2C0SDATN			(),
	.EMIOI2C1SCLO			(),
	.EMIOI2C1SCLTN			(),
	.EMIOI2C1SDAO			(),
	.EMIOI2C1SDATN			(),
	.EMIOPJTAGTDO			(),
	.EMIOPJTAGTDTN			(),
	.EMIOSDIO0BUSPOW		(),
	.EMIOSDIO0BUSVOLT		(),
	.EMIOSDIO0CLK			(),
	.EMIOSDIO0CMDO			(),
	.EMIOSDIO0CMDTN			(),
	.EMIOSDIO0DATAO			(),
	.EMIOSDIO0DATATN		(),
	.EMIOSDIO0LED			(),
	.EMIOSDIO1BUSPOW		(),
	.EMIOSDIO1BUSVOLT		(),
	.EMIOSDIO1CLK			(),
	.EMIOSDIO1CMDO			(),
	.EMIOSDIO1CMDTN			(),
	.EMIOSDIO1DATAO			(),
	.EMIOSDIO1DATATN		(),
	.EMIOSDIO1LED			(),
	.EMIOSPI0MO			(),
	.EMIOSPI0MOTN			(),
	.EMIOSPI0SCLKO			(),
	.EMIOSPI0SCLKTN			(),
	.EMIOSPI0SO			(),
	.EMIOSPI0SSNTN			(),
	.EMIOSPI0SSON			(),
	.EMIOSPI0STN			(),
	.EMIOSPI1MO			(),
	.EMIOSPI1MOTN			(),
	.EMIOSPI1SCLKO			(),
	.EMIOSPI1SCLKTN			(),
	.EMIOSPI1SO			(),
	.EMIOSPI1SSNTN			(),
	.EMIOSPI1SSON			(),
	.EMIOSPI1STN			(),
	.EMIOTRACECTL			(),
	.EMIOTRACEDATA			(),
	.EMIOTTC0WAVEO			(),
	.EMIOTTC1WAVEO			(),
	.EMIOUART0DTRN			(),
	.EMIOUART0RTSN			(),
	.EMIOUART0TX			(),
	.EMIOUART1DTRN			(),
	.EMIOUART1RTSN			(),
	.EMIOUART1TX			(),
	.EMIOUSB0PORTINDCTL		(),
	.EMIOUSB0VBUSPWRSELECT		(),
	.EMIOUSB1PORTINDCTL		(),
	.EMIOUSB1VBUSPWRSELECT		(),
	.EMIOWDTRSTO			(),
	.EVENTEVENTO			(),
	.EVENTSTANDBYWFE		(),
	.EVENTSTANDBYWFI		(),
	.FCLKCLK			(),
	.FCLKRESETN			(),
	.FTMTF2PTRIGACK			(),
	.FTMTP2FDEBUG			(),
	.FTMTP2FTRIG			(),
	.IRQP2F				(),
	.MAXIGP0ARADDR			(),
	.MAXIGP0ARBURST			(),
	.MAXIGP0ARCACHE			(),
	.MAXIGP0ARESETN			(),
	.MAXIGP0ARID			(),
	.MAXIGP0ARLEN			(),
	.MAXIGP0ARLOCK			(),
	.MAXIGP0ARPROT			(),
	.MAXIGP0ARQOS			(),
	.MAXIGP0ARSIZE			(),
	.MAXIGP0ARVALID			(),
	.MAXIGP0AWADDR			(),
	.MAXIGP0AWBURST			(),
	.MAXIGP0AWCACHE			(),
	.MAXIGP0AWID			(),
	.MAXIGP0AWLEN			(),
	.MAXIGP0AWLOCK			(),
	.MAXIGP0AWPROT			(),
	.MAXIGP0AWQOS			(),
	.MAXIGP0AWSIZE			(),
	.MAXIGP0AWVALID			(),
	.MAXIGP0BREADY			(),
	.MAXIGP0RREADY			(),
	.MAXIGP0WDATA			(),
	.MAXIGP0WID			(),
	.MAXIGP0WLAST			(),
	.MAXIGP0WSTRB			(),
	.MAXIGP0WVALID			(),
	.MAXIGP1ARADDR			(),
	.MAXIGP1ARBURST			(),
	.MAXIGP1ARCACHE			(),
	.MAXIGP1ARESETN			(),
	.MAXIGP1ARID			(),
	.MAXIGP1ARLEN			(),
	.MAXIGP1ARLOCK			(),
	.MAXIGP1ARPROT			(),
	.MAXIGP1ARQOS			(),
	.MAXIGP1ARSIZE			(),
	.MAXIGP1ARVALID			(),
	.MAXIGP1AWADDR			(),
	.MAXIGP1AWBURST			(),
	.MAXIGP1AWCACHE			(),
	.MAXIGP1AWID			(),
	.MAXIGP1AWLEN			(),
	.MAXIGP1AWLOCK			(),
	.MAXIGP1AWPROT			(),
	.MAXIGP1AWQOS			(),
	.MAXIGP1AWSIZE			(),
	.MAXIGP1AWVALID			(),
	.MAXIGP1BREADY			(),
	.MAXIGP1RREADY			(),
	.MAXIGP1WDATA			(),
	.MAXIGP1WID			(),
	.MAXIGP1WLAST			(),
	.MAXIGP1WSTRB			(),
	.MAXIGP1WVALID			(),
	.SAXIACPARESETN			(),
	.SAXIACPARREADY			(),
	.SAXIACPAWREADY			(),
	.SAXIACPBID			(),
	.SAXIACPBRESP			(),
	.SAXIACPBVALID			(),
	.SAXIACPRDATA			(),
	.SAXIACPRID			(),
	.SAXIACPRLAST			(),
	.SAXIACPRRESP			(),
	.SAXIACPRVALID			(),
	.SAXIACPWREADY			(),
	.SAXIGP0ARESETN			(),
	.SAXIGP0ARREADY			(),
	.SAXIGP0AWREADY			(),
	.SAXIGP0BID			(),
	.SAXIGP0BRESP			(),
	.SAXIGP0BVALID			(),
	.SAXIGP0RDATA			(),
	.SAXIGP0RID			(),
	.SAXIGP0RLAST			(),
	.SAXIGP0RRESP			(),
	.SAXIGP0RVALID			(),
	.SAXIGP0WREADY			(),
	.SAXIGP1ARESETN			(),
	.SAXIGP1ARREADY			(),
	.SAXIGP1AWREADY			(),
	.SAXIGP1BID			(),
	.SAXIGP1BRESP			(),
	.SAXIGP1BVALID			(),
	.SAXIGP1RDATA			(),
	.SAXIGP1RID			(),
	.SAXIGP1RLAST			(),
	.SAXIGP1RRESP			(),
	.SAXIGP1RVALID			(),
	.SAXIGP1WREADY			(),
	.SAXIHP0ARESETN			(),
	.SAXIHP0ARREADY			(),
	.SAXIHP0AWREADY			(),
	.SAXIHP0BID			(),
	.SAXIHP0BRESP			(),
	.SAXIHP0BVALID			(),
	.SAXIHP0RACOUNT			(),
	.SAXIHP0RCOUNT			(),
	.SAXIHP0RDATA			(),
	.SAXIHP0RID			(),
	.SAXIHP0RLAST			(),
	.SAXIHP0RRESP			(),
	.SAXIHP0RVALID			(),
	.SAXIHP0WACOUNT			(),
	.SAXIHP0WCOUNT			(),
	.SAXIHP0WREADY			(),
	.SAXIHP1ARESETN			(),
	.SAXIHP1ARREADY			(),
	.SAXIHP1AWREADY			(),
	.SAXIHP1BID			(),
	.SAXIHP1BRESP			(),
	.SAXIHP1BVALID			(),
	.SAXIHP1RACOUNT			(),
	.SAXIHP1RCOUNT			(),
	.SAXIHP1RDATA			(),
	.SAXIHP1RID			(),
	.SAXIHP1RLAST			(),
	.SAXIHP1RRESP			(),
	.SAXIHP1RVALID			(),
	.SAXIHP1WACOUNT			(),
	.SAXIHP1WCOUNT			(),
	.SAXIHP1WREADY			(),
	.SAXIHP2ARESETN			(),
	.SAXIHP2ARREADY			(),
	.SAXIHP2AWREADY			(),
	.SAXIHP2BID			(),
	.SAXIHP2BRESP			(),
	.SAXIHP2BVALID			(),
	.SAXIHP2RACOUNT			(),
	.SAXIHP2RCOUNT			(),
	.SAXIHP2RDATA			(),
	.SAXIHP2RID			(),
	.SAXIHP2RLAST			(),
	.SAXIHP2RRESP			(),
	.SAXIHP2RVALID			(),
	.SAXIHP2WACOUNT			(),
	.SAXIHP2WCOUNT			(),
	.SAXIHP2WREADY			(),
	.SAXIHP3ARESETN			(),
	.SAXIHP3ARREADY			(),
	.SAXIHP3AWREADY			(),
	.SAXIHP3BID			(),
	.SAXIHP3BRESP			(),
	.SAXIHP3BVALID			(),
	.SAXIHP3RACOUNT			(),
	.SAXIHP3RCOUNT			(),
	.SAXIHP3RDATA			(),
	.SAXIHP3RID			(),
	.SAXIHP3RLAST			(),
	.SAXIHP3RRESP			(),
	.SAXIHP3RVALID			(),
	.SAXIHP3WACOUNT			(),
	.SAXIHP3WCOUNT			(),
	.SAXIHP3WREADY			(),
	.DDRA				(),
	.DDRBA				(),
	.DDRCASB			(),
	.DDRCKE				(),
	.DDRCKN				(),
	.DDRCKP				(),
	.DDRCSB				(),
	.DDRDM				(),
	.DDRDQ				(),
	.DDRDQSN			(),
	.DDRDQSP			(),
	.DDRDRSTB			(),
	.DDRODT				(),
	.DDRRASB			(),
	.DDRVRN				(),
	.DDRVRP				(),
	.DDRWEB				(),
	.MIO				(),
	.PSCLK				(),
	.PSPORB				(),
	.PSSRSTB			(),
	.DDRARB				(%(dout)u),
	.DMA0ACLK			(),
	.DMA0DAREADY			(),
	.DMA0DRLAST			(),
	.DMA0DRTYPE			(),
	.DMA0DRVALID			(),
	.DMA1ACLK			(),
	.DMA1DAREADY			(),
	.DMA1DRLAST			(),
	.DMA1DRTYPE			(),
	.DMA1DRVALID			(),
	.DMA2ACLK			(),
	.DMA2DAREADY			(),
	.DMA2DRLAST			(),
	.DMA2DRTYPE			(),
	.DMA2DRVALID			(),
	.DMA3ACLK			(),
	.DMA3DAREADY			(),
	.DMA3DRLAST			(),
	.DMA3DRTYPE			(),
	.DMA3DRVALID			(),
	.EMIOCAN0PHYRX			(),
	.EMIOCAN1PHYRX			(),
	.EMIOENET0EXTINTIN		(),
	.EMIOENET0GMIICOL		(),
	.EMIOENET0GMIICRS		(),
	.EMIOENET0GMIIRXCLK		(),
	.EMIOENET0GMIIRXD		(),
	.EMIOENET0GMIIRXDV		(),
	.EMIOENET0GMIIRXER		(),
	.EMIOENET0GMIITXCLK		(),
	.EMIOENET0MDIOI			(),
	.EMIOENET1EXTINTIN		(),
	.EMIOENET1GMIICOL		(),
	.EMIOENET1GMIICRS		(),
	.EMIOENET1GMIIRXCLK		(),
	.EMIOENET1GMIIRXD		(),
	.EMIOENET1GMIIRXDV		(),
	.EMIOENET1GMIIRXER		(),
	.EMIOENET1GMIITXCLK		(),
	.EMIOENET1MDIOI			(),
	.EMIOGPIOI			(),
	.EMIOI2C0SCLI			(),
	.EMIOI2C0SDAI			(),
	.EMIOI2C1SCLI			(),
	.EMIOI2C1SDAI			(),
	.EMIOPJTAGTCK			(),
	.EMIOPJTAGTDI			(),
	.EMIOPJTAGTMS			(),
	.EMIOSDIO0CDN			(),
	.EMIOSDIO0CLKFB			(),
	.EMIOSDIO0CMDI			(),
	.EMIOSDIO0DATAI			(),
	.EMIOSDIO0WP			(),
	.EMIOSDIO1CDN			(),
	.EMIOSDIO1CLKFB			(),
	.EMIOSDIO1CMDI			(),
	.EMIOSDIO1DATAI			(),
	.EMIOSDIO1WP			(),
	.EMIOSPI0MI			(),
	.EMIOSPI0SCLKI			(),
	.EMIOSPI0SI			(),
	.EMIOSPI0SSIN			(),
	.EMIOSPI1MI			(),
	.EMIOSPI1SCLKI			(),
	.EMIOSPI1SI			(),
	.EMIOSPI1SSIN			(),
	.EMIOSRAMINTIN			(),
	.EMIOTRACECLK			(),
	.EMIOTTC0CLKI			(),
	.EMIOTTC1CLKI			(),
	.EMIOUART0CTSN			(),
	.EMIOUART0DCDN			(),
	.EMIOUART0DSRN			(),
	.EMIOUART0RIN			(),
	.EMIOUART0RX			(),
	.EMIOUART1CTSN			(),
	.EMIOUART1DCDN			(),
	.EMIOUART1DSRN			(),
	.EMIOUART1RIN			(),
	.EMIOUART1RX			(),
	.EMIOUSB0VBUSPWRFAULT		(),
	.EMIOUSB1VBUSPWRFAULT		(),
	.EMIOWDTCLKI			(),
	.EVENTEVENTI			(),
	.FCLKCLKTRIGN			(),
	.FPGAIDLEN			(),
	.FTMDTRACEINATID		(),
	.FTMDTRACEINCLOCK		(),
	.FTMDTRACEINDATA		(),
	.FTMDTRACEINVALID		(),
	.FTMTF2PDEBUG			(),
	.FTMTF2PTRIG			(),
	.FTMTP2FTRIGACK			(),
	.IRQF2P				(),
	.MAXIGP0ACLK			(),
	.MAXIGP0ARREADY			(),
	.MAXIGP0AWREADY			(),
	.MAXIGP0BID			(),
	.MAXIGP0BRESP			(),
	.MAXIGP0BVALID			(),
	.MAXIGP0RDATA			(),
	.MAXIGP0RID			(),
	.MAXIGP0RLAST			(),
	.MAXIGP0RRESP			(),
	.MAXIGP0RVALID			(),
	.MAXIGP0WREADY			(),
	.MAXIGP1ACLK			(),
	.MAXIGP1ARREADY			(),
	.MAXIGP1AWREADY			(),
	.MAXIGP1BID			(),
	.MAXIGP1BRESP			(),
	.MAXIGP1BVALID			(),
	.MAXIGP1RDATA			(),
	.MAXIGP1RID			(),
	.MAXIGP1RLAST			(),
	.MAXIGP1RRESP			(),
	.MAXIGP1RVALID			(),
	.MAXIGP1WREADY			(),
	.SAXIACPACLK			(),
	.SAXIACPARADDR			(),
	.SAXIACPARBURST			(),
	.SAXIACPARCACHE			(),
	.SAXIACPARID			(),
	.SAXIACPARLEN			(),
	.SAXIACPARLOCK			(),
	.SAXIACPARPROT			(),
	.SAXIACPARQOS			(),
	.SAXIACPARSIZE			(),
	.SAXIACPARUSER			(),
	.SAXIACPARVALID			(),
	.SAXIACPAWADDR			(),
	.SAXIACPAWBURST			(),
	.SAXIACPAWCACHE			(),
	.SAXIACPAWID			(),
	.SAXIACPAWLEN			(),
	.SAXIACPAWLOCK			(),
	.SAXIACPAWPROT			(),
	.SAXIACPAWQOS			(),
	.SAXIACPAWSIZE			(),
	.SAXIACPAWUSER			(),
	.SAXIACPAWVALID			(),
	.SAXIACPBREADY			(),
	.SAXIACPRREADY			(),
	.SAXIACPWDATA			(),
	.SAXIACPWID			(),
	.SAXIACPWLAST			(),
	.SAXIACPWSTRB			(),
	.SAXIACPWVALID			(),
	.SAXIGP0ACLK			(),
	.SAXIGP0ARADDR			(),
	.SAXIGP0ARBURST			(),
	.SAXIGP0ARCACHE			(),
	.SAXIGP0ARID			(),
	.SAXIGP0ARLEN			(),
	.SAXIGP0ARLOCK			(),
	.SAXIGP0ARPROT			(),
	.SAXIGP0ARQOS			(),
	.SAXIGP0ARSIZE			(),
	.SAXIGP0ARVALID			(),
	.SAXIGP0AWADDR			(),
	.SAXIGP0AWBURST			(),
	.SAXIGP0AWCACHE			(),
	.SAXIGP0AWID			(),
	.SAXIGP0AWLEN			(),
	.SAXIGP0AWLOCK			(),
	.SAXIGP0AWPROT			(),
	.SAXIGP0AWQOS			(),
	.SAXIGP0AWSIZE			(),
	.SAXIGP0AWVALID			(),
	.SAXIGP0BREADY			(),
	.SAXIGP0RREADY			(),
	.SAXIGP0WDATA			(),
	.SAXIGP0WID			(),
	.SAXIGP0WLAST			(),
	.SAXIGP0WSTRB			(),
	.SAXIGP0WVALID			(),
	.SAXIGP1ACLK			(),
	.SAXIGP1ARADDR			(),
	.SAXIGP1ARBURST			(),
	.SAXIGP1ARCACHE			(),
	.SAXIGP1ARID			(),
	.SAXIGP1ARLEN			(),
	.SAXIGP1ARLOCK			(),
	.SAXIGP1ARPROT			(),
	.SAXIGP1ARQOS			(),
	.SAXIGP1ARSIZE			(),
	.SAXIGP1ARVALID			(),
	.SAXIGP1AWADDR			(),
	.SAXIGP1AWBURST			(),
	.SAXIGP1AWCACHE			(),
	.SAXIGP1AWID			(),
	.SAXIGP1AWLEN			(),
	.SAXIGP1AWLOCK			(),
	.SAXIGP1AWPROT			(),
	.SAXIGP1AWQOS			(),
	.SAXIGP1AWSIZE			(),
	.SAXIGP1AWVALID			(),
	.SAXIGP1BREADY			(),
	.SAXIGP1RREADY			(),
	.SAXIGP1WDATA			(),
	.SAXIGP1WID			(),
	.SAXIGP1WLAST			(),
	.SAXIGP1WSTRB			(),
	.SAXIGP1WVALID			(),
	.SAXIHP0ACLK			(),
	.SAXIHP0ARADDR			(),
	.SAXIHP0ARBURST			(),
	.SAXIHP0ARCACHE			(),
	.SAXIHP0ARID			(),
	.SAXIHP0ARLEN			(),
	.SAXIHP0ARLOCK			(),
	.SAXIHP0ARPROT			(),
	.SAXIHP0ARQOS			(),
	.SAXIHP0ARSIZE			(),
	.SAXIHP0ARVALID			(),
	.SAXIHP0AWADDR			(),
	.SAXIHP0AWBURST			(),
	.SAXIHP0AWCACHE			(),
	.SAXIHP0AWID			(),
	.SAXIHP0AWLEN			(),
	.SAXIHP0AWLOCK			(),
	.SAXIHP0AWPROT			(),
	.SAXIHP0AWQOS			(),
	.SAXIHP0AWSIZE			(),
	.SAXIHP0AWVALID			(),
	.SAXIHP0BREADY			(),
	.SAXIHP0RDISSUECAP1EN		(),
	.SAXIHP0RREADY			(),
	.SAXIHP0WDATA			(),
	.SAXIHP0WID			(),
	.SAXIHP0WLAST			(),
	.SAXIHP0WRISSUECAP1EN		(),
	.SAXIHP0WSTRB			(),
	.SAXIHP0WVALID			(),
	.SAXIHP1ACLK			(),
	.SAXIHP1ARADDR			(),
	.SAXIHP1ARBURST			(),
	.SAXIHP1ARCACHE			(),
	.SAXIHP1ARID			(),
	.SAXIHP1ARLEN			(),
	.SAXIHP1ARLOCK			(),
	.SAXIHP1ARPROT			(),
	.SAXIHP1ARQOS			(),
	.SAXIHP1ARSIZE			(),
	.SAXIHP1ARVALID			(),
	.SAXIHP1AWADDR			(),
	.SAXIHP1AWBURST			(),
	.SAXIHP1AWCACHE			(),
	.SAXIHP1AWID			(),
	.SAXIHP1AWLEN			(),
	.SAXIHP1AWLOCK			(),
	.SAXIHP1AWPROT			(),
	.SAXIHP1AWQOS			(),
	.SAXIHP1AWSIZE			(),
	.SAXIHP1AWVALID			(),
	.SAXIHP1BREADY			(),
	.SAXIHP1RDISSUECAP1EN		(),
	.SAXIHP1RREADY			(),
	.SAXIHP1WDATA			(),
	.SAXIHP1WID			(),
	.SAXIHP1WLAST			(),
	.SAXIHP1WRISSUECAP1EN		(),
	.SAXIHP1WSTRB			(),
	.SAXIHP1WVALID			(),
	.SAXIHP2ACLK			(),
	.SAXIHP2ARADDR			(),
	.SAXIHP2ARBURST			(),
	.SAXIHP2ARCACHE			(),
	.SAXIHP2ARID			(),
	.SAXIHP2ARLEN			(),
	.SAXIHP2ARLOCK			(),
	.SAXIHP2ARPROT			(),
	.SAXIHP2ARQOS			(),
	.SAXIHP2ARSIZE			(),
	.SAXIHP2ARVALID			(),
	.SAXIHP2AWADDR			(),
	.SAXIHP2AWBURST			(),
	.SAXIHP2AWCACHE			(),
	.SAXIHP2AWID			(),
	.SAXIHP2AWLEN			(),
	.SAXIHP2AWLOCK			(),
	.SAXIHP2AWPROT			(),
	.SAXIHP2AWQOS			(),
	.SAXIHP2AWSIZE			(),
	.SAXIHP2AWVALID			(),
	.SAXIHP2BREADY			(),
	.SAXIHP2RDISSUECAP1EN		(),
	.SAXIHP2RREADY			(),
	.SAXIHP2WDATA			(),
	.SAXIHP2WID			(),
	.SAXIHP2WLAST			(),
	.SAXIHP2WRISSUECAP1EN		(),
	.SAXIHP2WSTRB			(),
	.SAXIHP2WVALID			(),
	.SAXIHP3ACLK			(),
	.SAXIHP3ARADDR			(),
	.SAXIHP3ARBURST			(),
	.SAXIHP3ARCACHE			(),
	.SAXIHP3ARID			(),
	.SAXIHP3ARLEN			(),
	.SAXIHP3ARLOCK			(),
	.SAXIHP3ARPROT			(),
	.SAXIHP3ARQOS			(),
	.SAXIHP3ARSIZE			(),
	.SAXIHP3ARVALID			(),
	.SAXIHP3AWADDR			(),
	.SAXIHP3AWBURST			(),
	.SAXIHP3AWCACHE			(),
	.SAXIHP3AWID			(),
	.SAXIHP3AWLEN			(),
	.SAXIHP3AWLOCK			(),
	.SAXIHP3AWPROT			(),
	.SAXIHP3AWQOS			(),
	.SAXIHP3AWSIZE			(),
	.SAXIHP3AWVALID			(),
	.SAXIHP3BREADY			(),
	.SAXIHP3RDISSUECAP1EN		(),
	.SAXIHP3RREADY			(),
	.SAXIHP3WDATA			(),
	.SAXIHP3WID			(),
	.SAXIHP3WLAST			(),
	.SAXIHP3WRISSUECAP1EN		(),
	.SAXIHP3WSTRB			(),
	.SAXIHP3WVALID			()
	);
    ''' % {
                'dut': 'site_name',
                'dout': isone
            })

    print("endmodule")
    write_params(params)


if __name__ == '__main__':
    run()



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
#
# Copyright (C) 2017-2020  The Project X-Ray Authors.
#
# Use of this source code is governed by a ISC-style
# license that can be found in the LICENSE file or at
# https://opensource.org/licenses/ISC
#
# SPDX-License-Identifier: ISC
import os
import random
random.seed(int(os.getenv("SEED"), 16))
from prjxray import util
from prjxray import verilog
from prjxray.verilog import vrandbit, vrandbits
import json


def gen_bram18():
    '''
    sample:
    "sites": {
        "RAMB18_X0Y50": "FIFO18E1",
        "RAMB18_X0Y51": "RAMB18E1",
        "RAMB36_X0Y25": "RAMBFIFO36E1"
    },
    '''
    for _tile_name, site_name, _site_type in sorted(util.get_roi().gen_sites(
        ['RAMB18E1', 'FIFO18E1'])):
        yield site_name


def gen_bram36():
    for _tile_name, site_name, _site_type in util.get_roi().gen_sites(
        ['RAMBFIFO36E1']):
        yield site_name


def gen_brams():
    '''
    Correctly assign a site to either bram36 or 2x bram18
    '''
    # XXX: mix 18 and 36?
    for site in gen_bram18():
        yield ('RAMB18E1', site)


def place_bram18(site, loci):
    ports = {
        'clk': 'clk',
        'din': 'din[  %d +: 8]' % (8 * loci, ),
        'dout': 'dout[  %d +: 8]' % (8 * loci, ),
    }

    write_modes = ["WRITE_FIRST", "READ_FIRST", "NO_CHANGE"]
    collisions = ["DELAYED_WRITE", "PERFORMANCE"]
    priorities = ["RSTREG", "REGCE"]

    # Datasheet says 72 is legal in some cases, but think its a copy paste error from BRAM36
    # also 0 and 36 aren't real sizes
    # Bias choice to 18 as its needed to solve certain bits quickly
    widths = [1, 2, 4, 9, 18, 18, 36]
    mode = '"TDP"'

    read_width_a = random.choice(widths)
    write_width_b = random.choice(widths)
    if read_width_a >= 36 or write_width_b >= 36:
        read_width_b = 0
        write_width_a = 0
        mode = '"SDP"'
        doa_reg = vrandbit()
        dob_reg = doa_reg
        write_modes = ["WRITE_FIRST", "READ_FIRST"]
        rstreg_priority_a = verilog.quote(random.choice(priorities))
        rstreg_priority_b = rstreg_priority_a
    else:
        read_width_b = random.choice(widths)
        write_width_a = random.choice(widths)
        doa_reg = vrandbit()
        dob_reg = vrandbit()
        rstreg_priority_a = verilog.quote(random.choice(priorities))
        rstreg_priority_b = verilog.quote(random.choice(priorities))

    params = {
        'LOC': verilog.quote(site),
        'IS_CLKARDCLK_INVERTED': vrandbit(),
        'IS_CLKBWRCLK_INVERTED': vrandbit(),
        'IS_ENARDEN_INVERTED': vrandbit(),
        'IS_ENBWREN_INVERTED': vrandbit(),
        'IS_RSTRAMARSTRAM_INVERTED': vrandbit(),
        'IS_RSTRAMB_INVERTED': vrandbit(),
        'IS_RSTREGARSTREG_INVERTED': vrandbit(),
        'IS_RSTREGB_INVERTED': vrandbit(),
        'RAM_MODE': mode,
        'WRITE_MODE_A': verilog.quote(random.choice(write_modes)),
        'WRITE_MODE_B': verilog.quote(random.choice(write_modes)),
        "DOA_REG": doa_reg,
        "DOB_REG": dob_reg,
        "SRVAL_A": vrandbits(18),
        "SRVAL_B": vrandbits(18),
        "INIT_A": vrandbits(18),
        "INIT_B": vrandbits(18),
        "READ_WIDTH_A": read_width_a,
        "READ_WIDTH_B": read_width_b,
        "WRITE_WIDTH_A": write_width_a,
        "WRITE_WIDTH_B": write_width_b,
        "RDADDR_COLLISION_HWCONFIG": verilog.quote(random.choice(collisions)),
        "RSTREG_PRIORITY_A": rstreg_priority_a,
        "RSTREG_PRIORITY_B": rstreg_priority_b,
    }

    return ('my_RAMB18E1', ports, params)


def main():
    brams = list(gen_brams())
    DUTN = len(brams)
    DIN_N = DUTN * 8
    DOUT_N = DUTN * 8

    verilog.top_harness(DIN_N, DOUT_N)

    f = open('params.jl', 'w')
    f.write('module,loc,params\n')
    print(
        'module roi(input clk, input [%d:0] din, output [%d:0] dout);' %
        (DIN_N - 1, DOUT_N - 1))

    for loci, (site_type, site) in enumerate(brams):
        modname, ports, params = {
            'RAMB18E1': place_bram18,
            #'RAMBFIFO36E1': place_bram36,
        }[site_type](site, loci)

        verilog.instance(modname, 'inst_%u' % loci, ports, params=params)

        j = {'module': modname, 'i': loci, 'params': params}
        f.write('%s\n' % (json.dumps(j)))
        print('')

    f.close()
    print(
        '''endmodule

    // ---------------------------------------------------------------------

    ''')

    # RAMB18E1
    print(
        '''
    module my_RAMB18E1 (input clk, input [7:0] din, output [7:0] dout);
        parameter LOC = "";
        parameter IS_CLKARDCLK_INVERTED = 1'b0;
        parameter IS_CLKBWRCLK_INVERTED = 1'b0;
        parameter IS_ENARDEN_INVERTED = 1'b0;
        parameter IS_ENBWREN_INVERTED = 1'b0;
        parameter IS_RSTRAMARSTRAM_INVERTED = 1'b0;
        parameter IS_RSTRAMB_INVERTED = 1'b0;
        parameter IS_RSTREGARSTREG_INVERTED = 1'b0;
        parameter IS_RSTREGB_INVERTED = 1'b0;
        parameter RAM_MODE = "TDP";
        parameter WRITE_MODE_A = "WRITE_FIRST";
        parameter WRITE_MODE_B = "WRITE_FIRST";
        parameter RDADDR_COLLISION_HWCONFIG = "DELAYED_WRITE";
        parameter RSTREG_PRIORITY_A = "RSTREG";
        parameter RSTREG_PRIORITY_B = "RSTREG";

        parameter DOA_REG = 1'b0;
        parameter DOB_REG = 1'b0;
        parameter SRVAL_A = 18'b0;
        parameter SRVAL_B = 18'b0;
        parameter INIT_A = 18'b0;
        parameter INIT_B = 18'b0;

        parameter READ_WIDTH_A = 0;
        parameter READ_WIDTH_B = 0;
        parameter WRITE_WIDTH_A = 0;
        parameter WRITE_WIDTH_B = 0;
        ''')
    print('''\
        (* LOC=LOC *)
        RAMB18E1 #(''')
    for i in range(8):
        print("            .INITP_%02X(256'b0)," % (i, ))
    print('')
    for i in range(0x40):
        print("            .INIT_%02X(256'b0)," % (i, ))
    print('')
    print(
        '''
                .IS_CLKARDCLK_INVERTED(IS_CLKARDCLK_INVERTED),
                .IS_CLKBWRCLK_INVERTED(IS_CLKBWRCLK_INVERTED),
                .IS_ENARDEN_INVERTED(IS_ENARDEN_INVERTED),
                .IS_ENBWREN_INVERTED(IS_ENBWREN_INVERTED),
                .IS_RSTRAMARSTRAM_INVERTED(IS_RSTRAMARSTRAM_INVERTED),
                .IS_RSTRAMB_INVERTED(IS_RSTRAMB_INVERTED),
                .IS_RSTREGARSTREG_INVERTED(IS_RSTREGARSTREG_INVERTED),
                .IS_RSTREGB_INVERTED(IS_RSTREGB_INVERTED),
                .RAM_MODE(RAM_MODE),
                .WRITE_MODE_A(WRITE_MODE_A),
                .WRITE_MODE_B(WRITE_MODE_B),

                .DOA_REG(DOA_REG),
                .DOB_REG(DOB_REG),
                .SRVAL_A(SRVAL_A),
                .SRVAL_B(SRVAL_B),
                .INIT_A(INIT_A),
                .INIT_B(INIT_B),

                .READ_WIDTH_A(READ_WIDTH_A),
                .READ_WIDTH_B(READ_WIDTH_B),
                .WRITE_WIDTH_A(WRITE_WIDTH_A),
                .WRITE_WIDTH_B(WRITE_WIDTH_B),

                .RDADDR_COLLISION_HWCONFIG(RDADDR_COLLISION_HWCONFIG),

                .RSTREG_PRIORITY_A(RSTREG_PRIORITY_A),
                .RSTREG_PRIORITY_B(RSTREG_PRIORITY_B)
            ) ram (
                .CLKARDCLK(din[0]),
                .CLKBWRCLK(din[1]),
                .ENARDEN(din[2]),
                .ENBWREN(din[3]),
                .REGCEAREGCE(din[4]),
                .REGCEB(din[5]),
                .RSTRAMARSTRAM(din[6]),
                .RSTRAMB(din[7]),
                .RSTREGARSTREG(din[0]),
                .RSTREGB(din[1]),
                .ADDRARDADDR(din[2]),
                .ADDRBWRADDR(din[3]),
                .DIADI(din[4]),
                .DIBDI(din[5]),
                .DIPADIP(din[6]),
                .DIPBDIP(din[7]),
                .WEA(1'b0),
                .WEBWE(din[1]),
                .DOADO(dout[0]),
                .DOBDO(dout[1]),
                .DOPADOP(dout[2]),
                .DOPBDOP(dout[3]));
    endmodule
    ''')


if __name__ == "__main__":
    main()



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
#
# Copyright (C) 2017-2020  The Project X-Ray Authors.
#
# Use of this source code is governed by a ISC-style
# license that can be found in the LICENSE file or at
# https://opensource.org/licenses/ISC
#
# SPDX-License-Identifier: ISC
import os
import re
import random
random.seed(int(os.getenv("SEED"), 16))
from prjxray import util
from prjxray.db import Database
from prjxray.grid_types import GridLoc

PCIE_INT_Y_RE = re.compile("PCIE_INT_INTERFACE.*X[0-9]+Y([0-9]+)")


def get_pcie_int_tiles(grid, pcie_loc):
    def get_site_at_loc(loc):
        gridinfo = grid.gridinfo_at_loc(loc)

        sites = list(gridinfo.sites.keys())

        if len(sites) and sites[0].startswith("SLICE"):
            return sites[0]

        return None

    pcie_int_tiles = list()

    for tile_name in sorted(grid.tiles()):
        if not tile_name.startswith("PCIE_INT_INTERFACE"):
            continue

        m = PCIE_INT_Y_RE.match(tile_name)

        assert m

        int_y = int(m.group(1))

        if int_y % 50 == 0:
            loc = grid.loc_of_tilename(tile_name)
            is_left = loc.grid_x < pcie_loc.grid_x

            if is_left:
                for i in range(1, loc.grid_x):
                    loc_grid_x = loc.grid_x - i

                    site = get_site_at_loc(GridLoc(loc_grid_x, loc.grid_y))

                    if site:
                        break
            else:
                _, x_max, _, _ = grid.dims()
                for i in range(1, x_max - loc.grid_x):
                    loc_grid_x = loc.grid_x + i

                    site = get_site_at_loc(GridLoc(loc_grid_x, loc.grid_y))

                    if site:
                        break

            pcie_int_tiles.append((tile_name, is_left, site))

    return pcie_int_tiles


def gen_sites():
    db = Database(util.get_db_root(), util.get_part())
    grid = db.grid()
    for tile_name in sorted(grid.tiles()):
        loc = grid.loc_of_tilename(tile_name)
        gridinfo = grid.gridinfo_at_loc(loc)

        for site_name, site_type in gridinfo.sites.items():
            if site_type in ['PCIE_2_1']:
                pcie_int_tiles = get_pcie_int_tiles(grid, loc)

                yield pcie_int_tiles, site_name


def write_params(params):
    pinstr = 'tile,val,site\n'
    for tile, (site, val) in sorted(params.items()):
        pinstr += '%s,%s,%s\n' % (tile, val, site)
    open('params.csv', 'w').write(pinstr)


def run():
    print('''
module top();
    ''')

    params = {}

    sites = list(gen_sites())
    for pcie_int_tiles, site_name in sites:
        left_side = None
        right_side = None

        for tile, is_left, site in pcie_int_tiles:
            isone = random.randint(0, 1)
            params[tile] = (site_name, isone)

            if is_left:
                left_side = site
            else:
                right_side = site

        assert left_side and right_side

        print(
            '''
wire [1:0] PLDIRECTEDLINKCHANGE;
wire [68:0] MIMTXRDATA;

(* KEEP, DONT_TOUCH, LOC = "{left}" *)
LUT1 left_lut_{left} (.O(MIMTXRDATA[0]));

(* KEEP, DONT_TOUCH, LOC = "{right}" *)
LUT1 right_lut_{right} (.O(PLDIRECTEDLINKCHANGE[0]));

(* KEEP, DONT_TOUCH, LOC = "{site}" *)
PCIE_2_1 pcie_2_1_{site} (
    .PLDIRECTEDLINKCHANGE(PLDIRECTEDLINKCHANGE),
    .MIMTXRDATA(MIMTXRDATA)
);'''.format(site=site_name, right=right_side, left=left_side))

    print("endmodule")
    write_params(params)


if __name__ == '__main__':
    run()



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
#
# Copyright (C) 2017-2020  The Project X-Ray Authors.
#
# Use of this source code is governed by a ISC-style
# license that can be found in the LICENSE file or at
# https://opensource.org/licenses/ISC
#
# SPDX-License-Identifier: ISC

from prjxray.segmaker import Segmaker
from prjxray import segmaker
from prjxray import verilog
import os
import json
import csv


def bitfilter(frame, word):
    if frame not in [26, 27]:
        return False
    return True

def main():
    print("Loading tags")
    segmk = Segmaker("design.bits")

    with open('params.json', 'r') as f:
        design = json.load(f)

        for d in design['tiles']:
            print("design: " + str(d))
            site = d['site']
            tile = d['tile']

            connection = d['CONNECTION']

            if site.startswith('STARTUP'):
                segmk.add_site_tag(site, 'USRCCLKO_CONNECTED', connection == "CLOCK")

    segmk.compile(bitfilter=bitfilter)
    segmk.write(allow_empty=True)

if __name__ == "__main__":
    main()
